Package: BioUtils
Title: Biological Data Analysis and Visualization
Version: 0.1.3
Authors@R: 
    person("Spencer", "Treadway", , "spencer.treadway08@gmail.com", role = c("aut", "cre"))
Description: Provides tools for the analysis and visualization of gene
    expression data from the NCBI Gene Expression Omnibus (GEO). Implements
    a complete workflow including data import, quality control, differential
    expression analysis, co-expression network analysis, pathway enrichment,
    and multi-gene biomarker discovery. Differential expression uses the
    empirical Bayes moderated t-statistic of Smyth (2004)
    <doi:10.2202/1544-6115.1027>. Gene set enrichment analysis follows
    Subramanian et al. (2005) <doi:10.1073/pnas.0506580102>. Multi-gene
    biomarker selection uses the LASSO method of Tibshirani (1996)
    <doi:10.1111/j.2517-6161.1996.tb02080.x>. Effect sizes are computed
    as Cohen's d following Cohen (1988).
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 3.1.2)
Suggests: knitr, rmarkdown, msigdbr, plotly
VignetteBuilder: knitr
Imports: tidyr, GEOquery, Biobase, limma, glmnet, ggplot2, pheatmap,
        grDevices, fgsea
URL: https://spencertreadway.github.io/BioUtils/,
        https://github.com/spencertreadway/BioUtils
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-18 23:18:01 UTC; spenc
Author: Spencer Treadway [aut, cre]
Maintainer: Spencer Treadway <spencer.treadway08@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-22 20:24:09 UTC
Built: R 4.6.0; ; 2026-05-25 04:10:02 UTC; unix

# Generated by roxygen2: do not edit by hand

export(adaptive.t.test)
export(adjust.pvalues)
export(analyze.gene)
export(build.analysis.df)
export(classify.effect.size)
export(compute.ci)
export(compute.effect.size)
export(correlation.heatmap.plot)
export(extract.expression)
export(find.probe.by.gene)
export(fit.lasso)
export(flag.biological.relevance)
export(gene.analysis.plot)
export(gene.correlation.matrix)
export(get.gene.expression)
export(get.gene.name)
export(load.geo.soft)
export(nonparametric.test)
export(pca.plot)
export(run.gsea)
export(run.limma.de)
export(volcano.plot)
importFrom(stats,cor)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,oneway.test)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,t.test)
importFrom(stats,var.test)
importFrom(stats,wilcox.test)
importFrom(utils,globalVariables)
importFrom(utils,head)

# Bioutils

## Simple Biological Utilities with Rust's [u8]

<p>Bioutils provides simple biological utilities including: 
    <ul>complete iupac and quality character sets</ul>
    <ul> Functions to check sequence validity and content (palindromes too!)</ul>
    <ul> Functions to download human and mouse Gencode reference files</ul>
    <ul> Functions to download fastq files</ul>
    <ul> Functions to replace N or gaps with pseudorandom nucleotides</ul>
    <ul> Functions to create new random IUPAC sequences</ul>
</p>

<p> Please take a look at the align example to get a full practical walkthrough!</p>

<li>Character sets include punctuation, are subdivided, and implemented in Rust's [u8] rather than bitset</li>
<li>Implementations are centered around [u8], although character sets are also provided as [&amp;str], hashset u8 and hashset &amp;str.
Check back as more functionality gets added!</li>
</ul>
<h2 id="quick-start" class="section-header"><a href="#quick-start">Quick Start</a></h2>
<div class="example-wrap"><pre class="rust rust-example-rendered"><p>//! Check out the align example for a full practical walkthrough from downloading files to finding read positions!

//! Examples for using checks:

//! use bioutils::charsets::*;
//! use bioutils::utils::*;
//! use bioutils::utils::check::CheckU8;
//!
//! let dna = b"ACTG";
//! let rna = b"ACUG";
//! let homopolymerN = b"NNNN";
//! let homopolymerA = b"AAAA";
//! let gapna = b"AC-G";
//! let nna = b"ACnG";
//! let quality = b"@ABC";
//!
//! assert!(homopolymerN.is_homopolymer());
//! assert!(homopolymerA.is_homopolymer_not_n());
//! assert!(homopolymerN.is_homopolymer_n());
//!
//! assert!(gapna.has_gap());
//! assert!(nna.has_n());
//! assert!(dna.is_iupac());
//! assert!(rna.is_basic_rna());
//!
//! assert!(quality.is_phred33());
//! assert!(quality.is_phred64());
//! assert!(quality.is_solexa());</p></pre></div>
</div><h2 id='modules' class='section-header'><a href="#modules">Modules</a></h2>
<table><tr class='module-item'><td><a class="mod" href="charsets/index.html" title='bioutils::charsets mod'>charsets</a></td><td class='docblock-short'><p>Numerous IUPAC character sets to either use directly or create your own mix and match</p></td></tr><tr class='module-item'><td><a class="mod" href="files/index.html" title='bioutils::files mod'>files</a></td><td class='docblock-short'><p>Download human and mouse Gencode references, download fastq sample files</p></td></tr><tr class='module-item'><td><a class="mod" href="references/index.html" title='bioutils::references mod'>references</a></td><td class='docblock-short'><p>Currently includes human NCBI gencode GRCh38. Automatically downloads the latest version of user's choice.</p>
</td></tr><tr class='module-item'><td><a class="mod" href="utils/index.html" title='bioutils::utils mod'>utils</a></td><td class='docblock-short'><p>Functions for sequence checks, pseudorandom replacement of N or gaps, and functions to create new pseudoranndom sequences</p></td></tr></table></section><section id="search" class="content hidden"></section><section class="footer"></section>

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE,
  cache     = FALSE,
  warning   = FALSE,
  message   = FALSE,
  fig.width = 7,
  fig.height = 5
)

## ----load---------------------------------------------------------------------
library(BioUtils)

eset <- load.geo.soft("", "GDS507", log.transform = TRUE)

## ----extract------------------------------------------------------------------
geo <- extract.expression(eset)

## ----pca----------------------------------------------------------------------
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")

## ----limma--------------------------------------------------------------------
de.results <- run.limma.de(geo, condition.col = "disease.state")

## ----inspect_de---------------------------------------------------------------
# Look at the top 10 most significantly differentially expressed probes
head(de.results[order(de.results$adj.P.Val), ], 10)

## ----volcano------------------------------------------------------------------
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)

## ----top_probes---------------------------------------------------------------
# Extract the 10 most significant probe IDs (row names of the TopTable)
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))

# Resolve probe IDs to human-readable gene names
top.genes <- get.gene.name(geo$gene, top.probes)

# Some probes on the array do not map to annotated genes, remove those
annotated.mask <- which(top.genes != "")
top.probes <- top.probes[annotated.mask]
top.genes <- top.genes[annotated.mask]

cat("Top differentially expressed genes:\n")
print(top.genes)

## ----single_prep--------------------------------------------------------------
# Retrieve expression values for all top probes as a matrix
expr.mat <- get.gene.expression(geo$expression, top.probes)

# Build the long-format analysis data frame
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)

## ----multi_plot---------------------------------------------------------------
# Faceted violin + boxplot across all top genes simultaneously
gene.analysis.plot(df.multi)

## ----single_gene--------------------------------------------------------------
# Subset to a single gene of interest
gene.of.interest <- get.gene.name(geo$gene, top.probes[1], use.symbols=TRUE)
df.single <- df.multi[which(df.multi$gene == gene.of.interest), ]

# Annotated single-gene visualization
gene.analysis.plot(df.single)

## ----analyze------------------------------------------------------------------
result <- analyze.gene(df.single)
cat(result$interpretation)

## ----interpret----------------------------------------------------------------
# Inspect individual components
cat("Effect size (Cohen's d):", round(result$effect.size, 3), "\n")
cat("Effect size class:      ", result$effect.size.class, "\n")
cat("95% CI:                 [",
    round(result$confidence.interval$lower, 3), ",",
    round(result$confidence.interval$upper, 3), "]\n")
cat("Parametric p-value:     ", signif(result$p.value, 3), "\n")
cat("Nonparametric p-value:  ", signif(result$nonparametric.p, 3), "\n")
cat("Robustness:             ", result$robustness, "\n")
cat("Biological assessment:  ", result$biological.relevance, "\n")

## ----gene_sets----------------------------------------------------------------
# Download Hallmark gene sets for Homo sapiens
hallmark.df <- msigdbr::msigdbr(species = "Homo sapiens", category = "H")

# Convert to a named list: pathway name -> character vector of gene symbols
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)

# Each element is a vector of gene symbols belonging to that pathway
cat("Number of Hallmark gene sets:", length(pathways), "\n")
cat("Example - first 6 genes in HALLMARK_HYPOXIA:\n")
print(head(pathways[["HALLMARK_HYPOXIA"]]))

## ----gsea---------------------------------------------------------------------
gsea.results <- run.gsea(de.results, geo$gene, pathways, min.size = 15, max.size = 500)

# Sort by adjusted p-value and inspect the top enriched pathways
gsea.results <- gsea.results[order(gsea.results$padj), ]
print(head(gsea.results[, c("pathway", "NES", "pval", "padj", "size")], 10))

## ----gsea_followup------------------------------------------------------------
# Extract the leading edge genes of the top enriched pathway
top.pathway <- gsea.results$pathway[1]
leading.genes <- unlist(gsea.results$leadingEdge[1])

cat("Top enriched pathway:", top.pathway, "\n")
cat("Leading edge genes:\n")
print(leading.genes)

# Find probes for the leading edge genes and perform single-gene deep-dives
leading.probes <- find.probe.by.gene(geo$gene, leading.genes)
leading.probes <- leading.probes[leading.probes != ""]

if(length(leading.probes) > 0)
{
  leading.expr <- get.gene.expression(geo$expression, leading.probes)
  df.leading <- build.analysis.df(leading.expr, geo$phenotype, geo$gene)
  gene.analysis.plot(df.leading)
}

## ----coexp--------------------------------------------------------------------
# Compute pairwise Pearson correlations between our top probes
cor.mat <- gene.correlation.matrix(geo$expression, top.probes, method = "pearson")

# Visualize with hierarchical clustering
correlation.heatmap.plot(cor.mat, gene.names = get.gene.name(geo$gene, top.probes, use.symbols=TRUE))

## ----lasso--------------------------------------------------------------------
# Encode disease state as a binary outcome variable
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)

# Fit LASSO logistic regression across all top probes simultaneously
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)

# Extract genes selected by LASSO at the 1-standard-error lambda
coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected  <- coef.mat[coef.mat[, 1] != 0, ]
print(selected)

## ----full_pipeline------------------------------------------------------------
library(BioUtils)

# == 1. Import =================================================================
eset <- load.geo.soft("", "GDS507", log.transform = TRUE)
geo <- extract.expression(eset)

# == 2. Quality Control ========================================================
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")

# == 3. Genome-Wide Differential Expression ====================================
de.results <- run.limma.de(geo, condition.col = "disease.state")
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)

# == 4. Select Top Candidates ==================================================
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))
top.genes <- get.gene.name(geo$gene, top.probes)
valid.mask <- which(top.genes != "")
top.probes <- top.probes[valid.mask]
top.genes <- top.genes[valid.mask]

# == 5. Build Analysis Data Frame ==============================================
expr.mat <- get.gene.expression(geo$expression, top.probes)
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)

# == 6. Multi-Gene Visualization ===============================================
gene.analysis.plot(df.multi)

# == 7. Single-Gene Deep-Dive ==================================================
df.single <- df.multi[which(df.multi$gene == get.gene.name(geo$gene, top.probes[1], use.symbols=TRUE)), ]
gene.analysis.plot(df.single)
result <- analyze.gene(df.single)
cat(result$interpretation)

# == 8. Co-expression Analysis =================================================
cor.mat <- gene.correlation.matrix(geo$expression, top.probes, method = "pearson")
correlation.heatmap.plot(cor.mat, gene.names = get.gene.name(geo$gene, top.probes, use.symbols=TRUE))

# == 9. Pathway Enrichment =====================================================
hallmark.df <- msigdbr::msigdbr(species = "Homo sapiens", category = "H")
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
gsea.results <- run.gsea(de.results, geo$gene, pathways)
gsea.results <- gsea.results[order(gsea.results$padj), ]
print(head(gsea.results[, c("pathway", "NES", "pval", "padj")], 10))

# == 10. Multi-Gene Predictive Modeling ========================================
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)
coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected <- coef.mat[coef.mat[, 1] != 0, ]
print(selected)

## ----setup, include=FALSE-----------------------------------------------------
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE,
  cache     = FALSE,
  warning   = FALSE,
  message   = FALSE,
  fig.width = 7,
  fig.height = 5
)

## ----load---------------------------------------------------------------------
library(BioUtils)
library(plotly)
library(msigdbr)
library(ggplot2)

geo <- extract.expression(
  load.geo.soft(accession = "GDS507", log.transform = TRUE)
)

## ----pca----------------------------------------------------------------------
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")

## ----de-----------------------------------------------------------------------
de.results <- run.limma.de(geo, condition.col = "disease.state")

sig <- de.results[de.results$adj.P.Val < 0.05 & abs(de.results$logFC) > 1, ]
cat("Significant DE probes (FDR < 0.05, |logFC| > 1): ", nrow(sig), "\n")
cat("Upregulated in RCC:                              ", sum(sig$logFC > 0), "\n")
cat("Downregulated in RCC:                            ", sum(sig$logFC < 0), "\n")

## ----volcano_static-----------------------------------------------------------
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)

## ----volcano_plotly-----------------------------------------------------------
de.plot <- de.results
de.plot$neg.log10.fdr <- -log10(de.plot$adj.P.Val)
de.plot$status <- "Not Significant"
de.plot$status[de.plot$adj.P.Val < 0.05 & de.plot$logFC > 1] <- "Upregulated"
de.plot$status[de.plot$adj.P.Val < 0.05 & de.plot$logFC < -1] <- "Downregulated"

de.plot$gene <- ""
top.idx <- order(de.plot$adj.P.Val)[1:50]
de.plot$gene[top.idx] <- get.gene.name(
  geo$gene, rownames(de.plot)[top.idx], use.symbols = TRUE
)

plotly::plot_ly(
  data = de.plot,
  x = ~logFC,
  y = ~neg.log10.fdr,
  color = ~status,
  colors = c("Upregulated" = "firebrick",
             "Downregulated" = "steelblue",
             "Not Significant" = "grey70"),
  text = ~paste0("Gene: ", gene,
                 "<br>logFC: ", round(logFC, 2),
                 "<br>FDR: ", signif(adj.P.Val, 3)),
  hoverinfo = "text",
  type = "scatter",
  mode = "markers",
  marker = list(size = 4, opacity = 0.6)
) %>%
  plotly::layout(
    title = "Volcano Plot: RCC vs Normal Kidney (Interactive, hover for gene labels)",
    xaxis = list(title = "Log2 Fold Change", zeroline = TRUE),
    yaxis = list(title = "-Log10 Adjusted P-Value", zeroline = FALSE),
    shapes = list(
      list(type = "line", x0 = 1,  x1 = 1,
           y0 = 0, y1 = max(de.plot$neg.log10.fdr, na.rm = TRUE),
           line = list(dash = "dot", color = "black")),
      list(type = "line", x0 = -1, x1 = -1,
           y0 = 0, y1 = max(de.plot$neg.log10.fdr, na.rm = TRUE),
           line = list(dash = "dot", color = "black"))
    )
  )

## ----top_candidates-----------------------------------------------------------
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))
top.symbols <- get.gene.name(geo$gene, top.probes, use.symbols = TRUE)
valid <- which(top.symbols != "")
top.probes <- top.probes[valid]
top.symbols <- top.symbols[valid]

expr.mat <- get.gene.expression(geo$expression, top.probes)
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)

gene.analysis.plot(df.multi)

## ----single-------------------------------------------------------------------
df.single <- df.multi[df.multi$gene == top.symbols[1], ]
gene.analysis.plot(df.single)
result <- analyze.gene(df.single)
cat(result$interpretation)

## ----coexp--------------------------------------------------------------------
cor.mat <- gene.correlation.matrix(
  geo$expression, top.probes, method = "pearson"
)
correlation.heatmap.plot(cor.mat, gene.names = top.symbols)

## ----gsea---------------------------------------------------------------------
hallmark.df <- msigdbr(species = "Homo sapiens", category = "H")
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
gsea.results <- run.gsea(de.results, geo$gene, pathways)
gsea.results <- gsea.results[order(gsea.results$padj), ]
top.pathways <- head(gsea.results[!is.na(gsea.results$padj), ], 15)

## ----gsea_plot, fig.height=7--------------------------------------------------
top.pathways$short.name <- gsub("HALLMARK_", "", top.pathways$pathway)
top.pathways$direction <- ifelse(top.pathways$NES > 0,
                                 "Upregulated in RCC",
                                 "Downregulated in RCC")

ggplot2::ggplot(
  top.pathways,
  ggplot2::aes(
    x = reorder(short.name, NES),
    y = NES,
    fill = direction
  )
) +
  ggplot2::geom_col() +
  ggplot2::coord_flip() +
  ggplot2::scale_fill_manual(
    values = c("Upregulated in RCC" = "firebrick",
               "Downregulated in RCC" = "steelblue")
  ) +
  ggplot2::geom_hline(yintercept = 0, linetype = "solid", color = "black") +
  ggplot2::labs(
    title = "Hallmark Pathway Enrichment in RCC vs Normal Kidney",
    subtitle = "Bars extend from zero; direction indicates enrichment in RCC (red) or normal (blue)",
    x = NULL,
    y = "Normalized Enrichment Score (NES)",
    fill = NULL
  ) +
  ggplot2::theme_minimal() +
  ggplot2::theme(
    plot.title = ggplot2::element_text(hjust = 0.5, size = 13),
    plot.subtitle = ggplot2::element_text(hjust = 0.5, size = 9,
                                          color = "grey40"),
    legend.position = "bottom"
  )

## ----lasso--------------------------------------------------------------------
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)

coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected <- coef.mat[coef.mat[, 1] != 0, , drop = FALSE]
n.selected <- nrow(selected) - 1  # subtract intercept

cat("Number of genes selected by LASSO:", n.selected, "\n\n")
print(round(selected, 4))

## ----lasso_plot---------------------------------------------------------------
# Visualize the cross-validation curve to show model selection
plot(lasso.fit, main = "LASSO Cross-Validation Curve")

#' Build Analysis Data Frame
#'
#' Combines a multi-gene expression matrix with sample metadata into a
#' long-format, analysis-ready data frame. Each row represents one gene-sample
#' observation, with human-readable gene names resolved from the annotation
#' data frame.
#'
#' @param expr.matrix Numeric matrix of gene expression values as returned by
#'   \code{get.gene.expression()}. Rows are probe IDs, columns are sample IDs.
#' @param phenotype Data frame of sample metadata as returned by
#'   \code{extract.expression()$phenotype}. Row names must correspond to the
#'   column names of \code{expr.matrix}.
#' @param genes Data frame of gene annotations as returned by
#'   \code{extract.expression()$gene}. Row indices must correspond to the probe
#'   IDs (row names) of \code{expr.matrix}. Used to resolve probe IDs to the
#'   human-readable gene names stored in the \code{"Gene symbol"} column.
#' @param group.col Character. Name of the column in \code{phenotype} to use as
#'   the grouping variable. Default is \code{"disease.state"}.
#'
#' @return A long-format data frame with one row per gene-sample pair and three
#'   columns:
#' \describe{
#'   \item{gene}{Character. Human-readable gene name resolved from the gene
#'     annotation data frame.}
#'   \item{expression}{Numeric. Expression value for that gene-sample pair.}
#'   \item{group}{Character or factor. Group label for each sample, renamed
#'     from the \code{group.col} column in \code{phenotype} for consistency
#'     with downstream functions.}
#' }
#' Rows with \code{NaN} or \code{NA} expression values are removed.
#'
#' @details
#' The function pivots \code{expr.matrix} from wide format (probes x samples)
#' to long format, merges sample metadata by sample ID, resolves probe IDs to
#' gene names using the annotation data frame, and selects the three columns
#' needed for analysis. The output is the standard input for
#' \code{analyze.gene()}, \code{gene.analysis.plot()}, and \code{fit.lasso()}.
#'
#' Requires \code{tidyr} for the pivot step. Ensure \code{tidyr} is listed
#' under \code{Imports} in the package DESCRIPTION.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#' probe <- find.probe.by.gene(geo$gene, c("MUC20", "ADH1A"))
#' expr <- get.gene.expression(geo$expression, probe)
#' df <- build.analysis.df(expr, geo$phenotype, geo$gene)
#' head(df)
#' }
#'
#' @export
build.analysis.df <- function(expr.matrix, phenotype, genes, group.col="disease.state")
{
  if(dim(expr.matrix)[2] == 1)
  {
    expr.matrix <- t(expr.matrix)
  }
  df <- as.data.frame(expr.matrix)
  df$probe <- rownames(df)

  long.df <- tidyr::pivot_longer(
    df,
    cols = -tidyr::all_of("probe"),
    names_to = "sample",
    values_to = "expression"
  )
  long.df <- merge(
    long.df,
    phenotype,
    by.x = "sample",
    by.y = "row.names"
  )

  # Resolve numeric probe IDs to human-readable gene names
  long.df$gene <- genes[genes$ID %in% long.df$probe, ][["Gene symbol"]]

  long.df <- long.df[!is.nan(long.df$expression), ]
  long.df <- na.omit(long.df)

  result <- long.df[, c("gene", "expression", group.col)]
  colnames(result)[3] <- "group"

  return(result)
}

#' Load GEO Dataset from SOFT File
#'
#' Imports a GEO dataset and converts it into a structured ExpressionSet object.
#' If a local SOFT file is found at \code{file.path} it is loaded directly.
#' Otherwise the dataset is downloaded from NCBI GEO using \code{accession} and
#' cached in \code{tempdir()}.
#'
#' @param file.path Character. Path to a local GEO SOFT file. If the file does
#'   not exist and \code{accession} is provided, the dataset is downloaded
#'   automatically.
#' @param accession Character. GEO accession ID (e.g., \code{"GDS507"}). Only
#'   used when \code{file.path} does not exist. Default is \code{NULL}.
#' @param log.transform Logical. Whether to apply a log2 transformation to the
#'   expression matrix during import. Default is \code{FALSE}.
#'
#' @return An \code{ExpressionSet} object for use with \code{extract.expression()}.
#'
#' @examples
#' \donttest{
#' # Download automatically if not found locally
#' eset <- load.geo.soft("GDS507.soft", accession = "GDS507", log.transform = TRUE)
#'
#' # Download without a local file at all
#' eset <- load.geo.soft(NULL, accession = "GDS507", log.transform = TRUE)
#' }
#'
#' @export
load.geo.soft <- function(file.path=NULL, accession=NULL, log.transform=FALSE)
{
  if(!is.null(file.path) && file.exists(file.path))
  {
    return(GEOquery::GDS2eSet(GEOquery::getGEO(filename=file.path),
                              do.log2=log.transform))
  }

  if(is.null(accession))
  {
    stop("File not found and no accession provided. ",
         "Supply an accession (e.g., \"GDS507\") to download automatically.")
  }

  message(accession, " not found locally, downloading from NCBI GEO...")
  gds <- GEOquery::getGEO(accession, destdir=tempdir(), GSEMatrix=FALSE)
  return(GEOquery::GDS2eSet(gds, do.log2=log.transform))
}

#' Extract Expression and Metadata from GEO Dataset
#'
#' Decomposes an \code{ExpressionSet} object into its core components,
#' returning a named list that is used as the primary data structure
#' throughout BioUtils.
#'
#' @param eset An \code{ExpressionSet} object as returned by
#'   \code{load.geo.soft()}.
#'
#' @return A named list with three elements:
#' \describe{
#'   \item{expression}{Numeric matrix of gene expression values. Rows are
#'     probe IDs, columns are sample IDs.}
#'   \item{phenotype}{Data frame of sample metadata (e.g., disease state,
#'     age, tissue). Row names are sample IDs matching the column names of
#'     \code{expression}.}
#'   \item{gene}{Data frame of probe-level gene annotations (e.g., gene
#'     symbol, gene title, chromosomal location). Row names are probe IDs
#'     matching the row names of \code{expression}.}
#' }
#'
#' @details
#' The three returned components form the basis of all downstream BioUtils
#' workflows. The \code{gene} data frame is passed to
#' \code{find.probe.by.gene()} for probe lookup, \code{expression} is
#' passed to \code{get.gene.expression()}, and \code{phenotype} is passed
#' to \code{build.analysis.df()} for group labeling.
#'
#' @examples
#' \donttest{
#' eset <- load.geo.soft(accession = "GDS507", log.transform = TRUE)
#' geo <- extract.expression(eset)
#' head(geo$phenotype)
#' dim(geo$expression)
#' }
#'
#' @export
extract.expression <- function(eset)
{
  if(!inherits(eset, "ExpressionSet"))
  {
    stop("Invalid argument: expected ExpressionSet, got ", class(eset))
  }

  gene.expression <- Biobase::exprs(eset)
  phenotype <- Biobase::pData(eset)
  genes <- Biobase::fData(eset)

  return(list(
    expression = gene.expression,
    phenotype = phenotype,
    gene = genes
  ))
}

#' Plot Gene Expression with Automatic Statistical Analysis
#'
#' Generates an annotated visualization of gene expression between two groups.
#' For a single-gene data frame, the plot is annotated with statistical results
#' from \code{analyze.gene()}. For a multi-gene data frame, the plot is
#' faceted by gene and annotations are omitted, use \code{analyze.gene()} on
#' each gene subset for per-gene statistics.
#'
#' @param df A data frame as produced by \code{build.analysis.df()}, containing:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector with exactly two group labels.}
#'   \item{gene}{Character vector of gene names. Optional; when present and
#'     containing more than one unique value, enables faceted multi-gene
#'     plotting.}
#' }
#'
#' @param alpha Numeric. Significance level passed to \code{analyze.gene()}.
#'   Default is \code{0.05}. Only used in single-gene mode.
#' @param n.boot Integer. Number of bootstrap resamples for the confidence
#'   interval, passed to \code{analyze.gene()}. Default is \code{1000}.
#'   Only used in single-gene mode.
#' @param show.points Logical. Whether to overlay jittered individual sample
#'   points on the violin/boxplot. Default is \code{TRUE}.
#'
#' @return A \code{ggplot} object. In single-gene mode the plot includes a
#'   text annotation with p-value, Cohen's d, and bootstrapped CI, and a
#'   subtitle with the full interpretation string. In multi-gene mode the
#'   plot is faceted by gene with no statistical annotation.
#'
#' @details
#' Single-gene mode is triggered when the \code{gene} column is absent or
#' contains exactly one unique value. Multi-gene mode is triggered when
#' \code{gene} contains more than one unique value, producing a
#' \code{facet_wrap(~ gene)} layout. The plot theme is consistent across
#' both modes and matches the style conventions of the BioUtils package.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#'
#' # Single-gene plot with statistical annotation
#' probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")
#' expr <- get.gene.expression(geo$expression, probe)
#' df <- build.analysis.df(expr, geo$phenotype, geo$gene)
#' gene.analysis.plot(df)
#'
#' # Multi-gene faceted plot
#' probes <- find.probe.by.gene(geo$gene, c(
#'   "mucin 20, cell surface associated",
#'   "alcohol dehydrogenase 1A (class I), alpha polypeptide"
#' ))
#' expr.multi <- get.gene.expression(geo$expression, probes)
#' df.multi <- build.analysis.df(expr.multi, geo$phenotype, geo$gene)
#' gene.analysis.plot(df.multi)
#' }
#'
#' @export
gene.analysis.plot <- function(df, alpha=0.05, n.boot=1000, show.points=TRUE)
{
  multi.gene <- "gene" %in% colnames(df) && length(unique(df$gene)) > 1

  # Base plot shared by both modes
  p <- ggplot2::ggplot(df, ggplot2::aes(x=group, y=expression)) +
    ggplot2::geom_violin(trim=FALSE, alpha=0.6) +
    ggplot2::geom_boxplot(width=0.1, outlier.shape=NA)

  if(show.points && !multi.gene)
  {
    p <- p + ggplot2::geom_jitter(width=0.15, alpha=0.4)
  }

  if(multi.gene)
  {
    p <- p +
      ggplot2::facet_wrap(~ gene, scales="free_y") +
      ggplot2::labs(
        title = "Gene Expression Comparison",
        x = "Group",
        y = "Expression"
      )
  }
  else
  {
    # Single-gene mode: run full statistical analysis and annotate
    result <- analyze.gene(df, alpha=alpha, n.boot=n.boot)
    ci <- result$confidence.interval

    label <- paste(
      "p = ", signif(result$p.value, 3),
      "\nd = ", round(result$effect.size, 2),
      " (", result$effect.size.class, ")",
      "\n", 1 - alpha, "% CI: [", signif(ci$lower, 2), ", ", signif(ci$upper, 2), "]"
    )

    p <- p +
      ggplot2::annotate(
        "text",
        x = 1.5,
        y = Inf,
        label = label,
        hjust = 0.5,
        vjust = 1.5,
        size = 4
      ) +
      ggplot2::labs(
        title = "Gene Expression Comparison",
        subtitle = result$interpretation,
        x = "Group",
        y = "Expression"
      )
  }

  p <- p + ggplot2::theme(
    panel.background = ggplot2::element_blank(),
    panel.border = ggplot2::element_rect(color="black"),
    panel.grid.major = ggplot2::element_blank(),
    axis.line = ggplot2::element_line(color="black", linewidth=1.2),
    title = ggplot2::element_text(size=20),
    axis.title = ggplot2::element_text(size=16),
    plot.title = ggplot2::element_text(hjust=0.5)
  )

  return(p)
}

#' Plot PCA of Sample Expression Profiles
#'
#' Reduces the dimensionality of the full expression matrix using Principal
#' Component Analysis and plots samples in the first two principal component
#' axes, colored by a phenotype variable. Useful for quality control,
#' batch effect detection, and exploratory assessment of sample clustering.
#'
#' @param expression.matrix Matrix of gene expression values. Rows are probes,
#'   columns are samples.
#' @param phenotype Data frame of sample metadata, as returned by
#'   \code{extract.expression()$phenotype}.
#' @param color.by Character. Column name in \code{phenotype} to use for
#'   point coloring. Default is \code{"disease.state"}.
#' @param scale Logical. Whether to scale probes to unit variance before PCA.
#'   Default is \code{TRUE}.
#'
#' @return A ggplot object showing samples as points in PC1 vs PC2 space,
#'   colored by the specified phenotype variable. The percentage of variance
#'   explained is shown on each axis label.
#'
#' @details
#' PCA is typically applied at the whole-dataset level before any gene
#' filtering, making it a useful first diagnostic after \code{load.geo.soft()}
#' and \code{extract.expression()}. Tight clustering of samples by disease
#' state in PC1/PC2 space suggests a strong and detectable expression signal.
#' Unexpected clustering by batch, sex, or other covariates can indicate the
#' need for covariate adjustment in downstream \code{run.limma.de()} models.
#'
#' @examples
#' \donttest{
#' # Create a synthetic expression matrix (50 probes x 12 samples)
#' set.seed(42)
#' expr.mat <- matrix(rnorm(600), nrow = 50, ncol = 12)
#' rownames(expr.mat) <- paste0("probe", 1:50)
#' colnames(expr.mat) <- paste0("sample", 1:12)
#'
#' # Create a matching phenotype data frame
#' phenotype <- data.frame(
#'   disease.state = rep(c("normal", "RCC"), each = 6),
#'   row.names = paste0("sample", 1:12)
#' )
#'
#' pca.plot(expr.mat, phenotype, color.by = "disease.state")
#' }
#'
#' @export
pca.plot <- function(expression.matrix, phenotype, color.by="disease.state", scale=TRUE)
{
  pca <- prcomp(t(expression.matrix), scale.=scale)
  var.explained <- round(summary(pca)$importance[2, 1:2] * 100, 1)

  df <- data.frame(
    PC1 = pca$x[, 1],
    PC2 = pca$x[, 2],
    group = phenotype[[color.by]]
  )

  ggplot2::ggplot(df, ggplot2::aes(x=PC1, y=PC2, color=group)) +
    ggplot2::geom_point(size=3, alpha=0.8) +
    ggplot2::labs(
      title = "PCA of Sample Expression Profiles",
      x = paste0("PC1 (", var.explained[1], "% variance)"),
      y = paste0("PC2 (", var.explained[2], "% variance)"),
      color = color.by
    ) +
    ggplot2::theme(
      panel.background = ggplot2::element_blank(),
      panel.border = ggplot2::element_rect(color="black"),
      panel.grid.major = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(color="black", linewidth=1.2),
      title = ggplot2::element_text(size=20),
      axis.title = ggplot2::element_text(size=16),
      plot.title = ggplot2::element_text(hjust=0.5)
    )
}

#' Plot Volcano Plot from Differential Expression Results
#'
#' Generates a volcano plot visualizing the relationship between statistical
#' significance and fold-change magnitude for all probes in a differential
#' expression analysis. Points are colored by whether they exceed user-defined
#' significance and fold-change thresholds.
#'
#' @param de.results Data frame as returned by \code{run.limma.de()}, containing
#'   at minimum the columns \code{logFC} and \code{adj.P.Val}.
#' @param fc.threshold Numeric. Absolute log2 fold-change cutoff for labeling
#'   genes as differentially expressed. Default is \code{1} (i.e., 2-fold).
#' @param fdr.threshold Numeric. Adjusted p-value cutoff. Default is \code{0.05}.
#'
#' @return A ggplot object showing each probe as a point with \code{logFC} on
#'   the x-axis and \code{-log10(adj.P.Val)} on the y-axis. Points are colored
#'   as upregulated, downregulated, or not significant. Dashed threshold lines
#'   are drawn at the specified cutoffs.
#'
#' @details
#' The volcano plot is the standard summary visualization for a TopTable from
#' \code{run.limma.de()} and serves as the genome-wide complement to
#' \code{gene.analysis.plot()}, which visualizes a single gene. Probes that
#' appear in the upper corners (high fold-change, low FDR) are the strongest
#' candidates for follow-up with \code{analyze.gene()} or inclusion in a gene
#' signature for \code{run.gsea()}.
#'
#' @examples
#' \donttest{
#' \donttest{
#' # Synthesize a minimal TopTable as returned by run.limma.de()
#' set.seed(42)
#' n <- 200
#' de.results <- data.frame(
#'   logFC = rnorm(n, mean = 0, sd = 2),
#'   adj.P.Val = runif(n, min = 0, max = 1),
#'   AveExpr = rnorm(n, mean = 8, sd = 1),
#'   row.names = paste0("probe", 1:n)
#' )
#' volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)
#' }
#'
#' @export
volcano.plot <- function(de.results, fc.threshold=1, fdr.threshold=0.05)
{
  de.results$neg.log10.fdr <- -log10(de.results$adj.P.Val)
  de.results$status <- "Not Significant"
  de.results$status[de.results$adj.P.Val < fdr.threshold &
                      de.results$logFC > fc.threshold] <- "Upregulated"
  de.results$status[de.results$adj.P.Val < fdr.threshold &
                      de.results$logFC < -fc.threshold] <- "Downregulated"

  ggplot2::ggplot(de.results, ggplot2::aes(x=logFC, y=neg.log10.fdr, color=status)) +
    ggplot2::geom_point(alpha=0.5, size=1.5) +
    ggplot2::geom_vline(xintercept = c(-fc.threshold, fc.threshold), linetype = "dashed") +
    ggplot2::geom_hline(yintercept = -log10(fdr.threshold), linetype = "dashed") +
    ggplot2::scale_color_manual(values = c(
      "Upregulated" = "firebrick",
      "Downregulated" = "steelblue",
      "Not Significant" = "grey60"
    )) +
    ggplot2::labs(
      title = "Volcano Plot of Differential Expression",
      x = "Log2 Fold Change",
      y = "-Log10 Adjusted P-Value",
      color = "Status"
    ) +
    ggplot2::theme(
      panel.background = ggplot2::element_blank(),
      panel.border = ggplot2::element_rect(color="black"),
      panel.grid.major = ggplot2::element_blank(),
      axis.line = ggplot2::element_line(color="black", linewidth=1.2),
      title = ggplot2::element_text(size=20),
      axis.title = ggplot2::element_text(size=16),
      plot.title = ggplot2::element_text(hjust=0.5)
    )
}

#' Plot Gene Co-expression Correlation Heatmap
#'
#' Visualizes a gene-by-gene correlation matrix as a clustered heatmap,
#' revealing groups of co-expressed genes within a candidate gene set.
#'
#' @param cor.matrix Numeric matrix as returned by \code{gene.correlation.matrix()}.
#'   Must be square and symmetric with row/column names corresponding to probe IDs.
#' @param gene.names Character vector of gene names to use as axis labels in
#'   place of probe IDs. Must be the same length and order as \code{cor.matrix}
#'   rows. Default is \code{NULL}, which uses probe IDs.
#'
#' @return A \code{pheatmap} object displaying the correlation matrix with
#'   hierarchical clustering applied to both rows and columns. Color scale
#'   runs from blue (strong negative correlation) through white (no correlation)
#'   to red (strong positive correlation).
#'
#' @details
#' Hierarchical clustering of the correlation matrix groups genes with similar
#' co-expression patterns into visible blocks on the heatmap. These blocks
#' often correspond to genes in the same pathway or under shared regulatory
#' control. This function is a targeted companion to \code{gene.correlation.matrix()}
#' for a pre-selected gene set, and complements the genome-wide network view
#' produced by WGCNA.
#'
#' @examples
#' \donttest{
#' mat <- matrix(
#'   c(1.00, 0.85, 0.62, 0.91,
#'     0.85, 1.00, 0.74, 0.88,
#'     0.62, 0.74, 1.00, 0.69,
#'     0.91, 0.88, 0.69, 1.00),
#'   nrow = 4,
#'   dimnames = list(
#'     c("BRCA1", "TP53", "MYC", "EGFR"),
#'     c("BRCA1", "TP53", "MYC", "EGFR")
#'   )
#' )
#' correlation.heatmap.plot(mat, gene.names = c("BRCA1", "TP53", "MYC", "EGFR"))
#' }
#'
#' @export
correlation.heatmap.plot <- function(cor.matrix, gene.names=NULL)
{
  if (!is.null(gene.names))
  {
    rownames(cor.matrix) <- gene.names
    colnames(cor.matrix) <- gene.names
  }

  pheatmap::pheatmap(
    cor.matrix,
    color = grDevices::colorRampPalette(c("steelblue", "white", "firebrick"))(100),
    breaks = seq(-1, 1, length.out = 101),
    clustering_method = "complete",
    display_numbers = TRUE,
    number_format = "%.2f",
    angle_col = 45,
    angle_row = 0,
    main = "Gene Co-expression Correlation Heatmap"
  )
}

#' Find Probe IDs by Gene Name
#'
#' Searches the gene annotation data frame to find the probe IDs corresponding
#' to one or more gene names or descriptions.
#'
#' @param genes Data frame of gene annotations as returned by
#'   \code{extract.expression()$gene}. The second column (index 2) is expected
#'   to contain gene titles or descriptions.
#' @param gene.names Character vector of one or more gene names or descriptions
#'   to search for. Matching is exact and case-sensitive.
#'
#' @return Integer vector of probe IDs corresponding to the matched genes.
#'   The length of the vector equals the number of matches found. Returns an
#'   empty integer vector if no matches are found.
#'
#' @details
#' Probe IDs are the integer row names of the \code{genes} annotation data
#' frame. These IDs are used directly to index rows of the expression matrix
#' in \code{get.gene.expression()}. If multiple gene names are supplied, all
#' matching probe IDs are returned as a vector, which is then passed as-is to
#' \code{get.gene.expression()} to retrieve a multi-row expression matrix.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#'
#' # Single gene
#' probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")
#'
#' # Multiple genes
#' probes <- find.probe.by.gene(geo$gene, c(
#'   "mucin 20, cell surface associated",
#'   "alcohol dehydrogenase 1A (class I), alpha polypeptide"
#' ))
#' }
#'
#' @export
find.probe.by.gene <- function(genes, gene.names)
{
  return(as.integer(rownames(genes[which(genes[, 2] %in% gene.names), ])))
}

#' Extract Gene Expression by Probe ID
#'
#' Retrieves expression values for one or more genes from the expression matrix
#' using their integer probe IDs.
#'
#' @param expression Numeric matrix of gene expression values as returned by
#'   \code{extract.expression()$expression}. Rows are probe IDs, columns are
#'   sample IDs.
#' @param probe.id Integer vector of one or more probe IDs as returned by
#'   \code{find.probe.by.gene()}.
#'
#' @return A numeric matrix with one row per probe and one column per sample.
#'   Row names are the probe IDs; column names are the sample IDs. When a
#'   single probe ID is supplied the result is still a matrix (not a vector),
#'   ensuring that \code{build.analysis.df()} receives a consistent input type.
#'
#' @details
#' The returned matrix is the direct input for \code{build.analysis.df()}.
#' Using \code{as.matrix()} ensures the single-probe case returns a one-row
#' matrix rather than a named vector, which keeps the \code{pivot_longer}
#' step in \code{build.analysis.df()} consistent regardless of how many genes
#' are queried.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#' probes <- find.probe.by.gene(geo$gene, c(
#'   "mucin 20, cell surface associated",
#'   "alcohol dehydrogenase 1A (class I), alpha polypeptide"
#' ))
#' expr <- get.gene.expression(geo$expression, probes)
#' dim(expr)  # probes x samples
#' }
#'
#' @export
get.gene.expression <- function(expression, probe.id)
{
  return(as.matrix(expression[probe.id, ]))
}

#' Resolve Probe IDs to Gene Names
#'
#' Converts one or more integer probe IDs back to their human-readable gene
#' title strings or short gene symbols using the gene annotation data frame.
#' This is the inverse operation of \code{find.probe.by.gene()}.
#'
#' @param genes Data frame of gene annotations as returned by
#'   \code{extract.expression()$gene}. Must contain columns \code{"ID"},
#'   \code{"Gene title"}, and \code{"Gene symbol"}.
#' @param probe.id Integer or character vector of one or more probe IDs to
#'   resolve, as returned by \code{find.probe.by.gene()}.
#' @param use.symbols Logical. If \code{FALSE} (default), returns full
#'   descriptive gene titles from the \code{"Gene title"} column, suitable
#'   for interpretation and reporting. If \code{TRUE}, returns short gene
#'   symbols from the \code{"Gene symbol"} column, suitable for plot axis
#'   labels and any downstream tool that expects standard gene symbols such
#'   as \code{run.gsea()}.
#'
#' @return Character vector of gene names corresponding to the supplied probe
#'   IDs. Returns empty strings \code{""} for probes with no annotation, which
#'   should be filtered with \code{which(result != "")}.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS507",
#'                                         log.transform = TRUE))
#' de.results <- run.limma.de(geo)
#' top.probes <- rownames(head(de.results, 10))
#'
#' # Full titles for reporting
#' top.titles  <- get.gene.name(geo$gene, top.probes)
#'
#' # Short symbols for plot labels and GSEA
#' top.symbols <- get.gene.name(geo$gene, top.probes, use.symbols = TRUE)
#' }
#'
#' @export
get.gene.name <- function(genes, probe.id, use.symbols=FALSE)
{
  col <- if(use.symbols) "Gene symbol" else "Gene title"
  return(genes[which(genes$ID %in% probe.id), ][[col]])
}

#' BioUtils: Tools for Biological Data Analysis and Visualization
#'
#' BioUtils provides a collection of tools for preprocessing biological
#' datasets, performing statistical analyses, and generating visualizations.
#' See \code{vignette("bioutils-case-study")} for a full workflow walkthrough.
#'
#' @keywords internal
#' @importFrom stats cor model.matrix na.omit oneway.test p.adjust prcomp quantile sd t.test var.test wilcox.test
#' @importFrom utils head globalVariables
"_PACKAGE"

utils::globalVariables(c(
  # build.analysis.df — tidyselect column reference
  "probe",
  # stats_tests.R / visualization.R — ggplot2 aes() and formula objects
  "expression", "group", "gene", "ID",
  # pca.plot — columns created from prcomp output before aes()
  "PC1", "PC2",
  # volcano.plot — columns created on de.results before aes()
  "logFC", "neg.log10.fdr", "status"
))

#' Compute Effect Size (Cohen's d)
#'
#' Calculates the standardized mean difference between two groups.
#'
#' @param df A data frame containing at least two columns:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector with exactly two group labels,
#'     as produced by \code{build.analysis.df()}.}
#' }
#'
#' @return Numeric. Cohen's d value. Positive values indicate the first
#'   group (alphabetically) has higher mean expression; negative values
#'   indicate the second group is higher.
#'
#' @export
compute.effect.size <- function(df)
{
  groups <- split(df$expression, df$group)

  if(length(groups) < 2)
  {
    return(NA)
  }

  g1 <- groups[[1]]
  g2 <- groups[[2]]

  mean1 <- mean(g1)
  mean2 <- mean(g2)

  sd1 <- sd(g1)
  sd2 <- sd(g2)

  n1 <- length(g1)
  n2 <- length(g2)

  pooled.sd <- sqrt(((n1 - 1)*sd1^2 + (n2 - 1)*sd2^2) / (n1 + n2 - 2))
  d <- (mean1 - mean2) / pooled.sd

  return(d)
}

#' Compute Confidence Interval for Effect Size
#'
#' Estimates a confidence interval for Cohen's d using bootstrap resampling.
#' This provides a measure of uncertainty around the effect size estimate,
#' which is especially useful when sample sizes are small or distributions
#' deviate from normality.
#'
#' @param df A data frame containing at least two columns:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector indicating group membership.}
#' }
#' @param alpha Numeric. Significance level used to compute the confidence
#'   interval bounds. Default is \code{0.05}, yielding a 95 percent CI.
#'
#' @param n.boot Integer. Number of bootstrap resamples to perform.
#'   Default is \code{1000}.
#'
#' @return A named list with two elements:
#' \describe{
#'   \item{lower}{Lower bound of the bootstrapped confidence interval.}
#'   \item{upper}{Upper bound of the bootstrapped confidence interval.}
#' }
#'
#' @details
#' The function repeatedly resamples the input data with replacement and
#' recomputes Cohen's d for each resample. The confidence interval is
#' derived from the empirical distribution of bootstrapped effect sizes
#' using the percentile method.
#'
#' @examples
#' \donttest{
#' # Build a minimal example data frame
#' analysis.df <- data.frame(
#'   expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7),
#'   group = rep(c("normal", "RCC"), each = 4)
#' )
#' ci <- compute.ci(analysis.df, alpha = 0.05, n.boot = 100)
#' }
#'
#' @export
compute.ci <- function(df, alpha=0.05, n.boot=1000)
{
  boot.ds <- numeric(n.boot)

  for(i in 1:n.boot)
  {
    sample.df <- df[sample(nrow(df), replace=TRUE), ]
    boot.ds[i] <- compute.effect.size(sample.df)
  }

  boot.ds <- boot.ds[!is.na(boot.ds)]

  ci <- quantile(boot.ds, c(alpha / 2, 1 - (alpha / 2)), na.rm=TRUE)

  return(list(
    lower = ci[1],
    upper = ci[2]
  ))
}

#' Perform Nonparametric Group Comparison
#'
#' Conducts a Wilcoxon rank-sum test (Mann-Whitney U test) to compare
#' expression values between two groups without assuming normality.
#'
#' @param df A data frame containing at least two columns:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector with exactly two group labels.}
#' }
#'
#' @return A named list with two elements:
#' \describe{
#'   \item{p.value}{Numeric. P-value from the Wilcoxon rank-sum test.}
#'   \item{test}{Character. Name of the test used (\code{"Wilcoxon rank-sum"}).}
#' }
#'
#' @details
#' This test is useful as a robustness check against parametric methods
#' such as the t-test, especially when data are skewed or contain outliers.
#' The result is used by \code{analyze.gene()} to populate the
#' \code{robustness} field of its return value.
#'
#' @examples
#' \donttest{
#' analysis.df <- data.frame(
#'   expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7,
#'                  1.5, 2.0, 1.9, 2.4),
#'   group      = rep(c("normal", "RCC"), each = 6)
#' )
#' res <- nonparametric.test(analysis.df)
#' res$p.value
#' }
#'
#' @export
nonparametric.test <- function(df)
{
  res <- wilcox.test(expression ~ group, data=df)

  return(list(
    p.value = res$p.value,
    test = "Wilcoxon rank-sum"
  ))
}

#' Classify Effect Size
#'
#' Categorizes an absolute Cohen's d value into a qualitative magnitude label
#' using conventional thresholds.
#'
#' @param d Numeric. Cohen's d effect size value (signed or unsigned).
#'
#' @return Character string. One of \code{"negligible"}, \code{"small"},
#'   \code{"moderate"}, or \code{"large"} based on the absolute value of
#'   \code{d}, using the thresholds: negligible < 0.2 <= small < 0.5 <=
#'   moderate < 0.8 <= large.
#'
#' @examples
#' \donttest{
#' classify.effect.size(0.3)   # "small"
#' classify.effect.size(-0.9)  # "large"
#' }
#'
#' @export
classify.effect.size <- function(d)
{
  abs.d <- abs(d)

  if(abs.d < 0.2)
  {
    return("negligible")
  }
  else if(abs.d < 0.5)
  {
    return("small")
  }
  else if(abs.d < 0.8)
  {
    return("moderate")
  }
  else
  {
    return("large")
  }
}

#' Assess Biological Relevance of Gene Expression Differences
#'
#' Provides a heuristic interpretation of whether a statistically significant
#' difference in gene expression is likely to be biologically meaningful,
#' based on effect size magnitude and p-value.
#'
#' @param effect.size Numeric. Cohen's d or other standardized effect size.
#' @param p.value Numeric. P-value from a statistical test.
#' @param alpha Numeric. Significance threshold used for the p-value
#'   comparison. Default is \code{0.05}.
#'
#' @return Character string. One of three heuristic labels:
#' \describe{
#'   \item{"Potentially biologically meaningful"}{p < alpha and |d| > 0.5}
#'   \item{"Statistically significant but small effect"}{p < alpha and |d| <= 0.5}
#'   \item{"No strong evidence of biological relevance"}{p >= alpha}
#' }
#'
#' @details
#' These rules are heuristic and intended as guidance rather than definitive
#' biological conclusions. The thresholds for effect size (0.5) and
#' significance are conventional starting points; domain knowledge should
#' inform interpretation. This function is called internally by
#' \code{analyze.gene()} using the same \code{alpha} passed to that function.
#'
#' @examples
#' \donttest{
#' flag.biological.relevance(effect.size = 0.6, p.value = 0.01)
#' }
#'
#' @export
flag.biological.relevance <- function(effect.size, p.value, alpha=0.05)
{
  if(p.value < alpha && abs(effect.size) > 0.5)
  {
    return("Potentially biologically meaningful")
  }

  if(p.value < alpha && abs(effect.size) <= 0.5)
  {
    return("Statistically significant but small effect")
  }

  return("No strong evidence of biological relevance")
}

#' Perform Adaptive T-Test
#'
#' Automatically selects between Welch's and Student's t-test based on the
#' result of a variance equality test, then returns a unified result structure
#' regardless of which branch was taken.
#'
#' @param df A data frame containing at least two columns:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector with exactly two group labels.}
#' }
#' @param alpha Numeric. Significance level for the variance equality test
#'   (\code{var.test()}). Default is \code{0.05}.
#'
#' @return A named list with three elements:
#' \describe{
#'   \item{variance.test}{The result object from \code{var.test()} or
#'     \code{oneway.test()}, depending on the branch taken.}
#'   \item{t.test}{The result object from \code{t.test()}.}
#'   \item{p.value}{Numeric. The p-value from the t-test.}
#' }
#'
#' @details
#' If \code{var.test()} returns a p-value below \code{alpha}, variances are
#' considered unequal and Welch's t-test (\code{var.equal = FALSE}) is used.
#' Otherwise, Student's t-test (\code{var.equal = TRUE}) is applied alongside
#' a one-way ANOVA as a confirmatory check. The returned list is consumed
#' internally by \code{analyze.gene()}.
#'
#' @examples
#' \donttest{
#' # Build a minimal example data frame
#' analysis.df <- data.frame(
#'   expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7),
#'   group = rep(c("normal", "RCC"), each = 4)
#' )
#' result <- adaptive.t.test(analysis.df, alpha = 0.05)
#' result$p.value
#' }
#'
#' @export
adaptive.t.test <- function(df, alpha=0.05)
{
  variance.result <- var.test(expression ~ group, data=df)

  if(variance.result$p.value < alpha)
  {
    t.result <- t.test(expression ~ group, data=df, var.equal=FALSE, conf.level=1-alpha)

    return(list(
      variance.test = variance.result,
      t.test = t.result,
      p.value = t.result$p.value
    ))
  }
  else
  {
    t.result <- t.test(expression ~ group, data=df, var.equal=TRUE, conf.level=1-alpha)
    anova.result <- oneway.test(expression ~ group, data=df, var.equal=TRUE)

    return(list(
      variance.test = anova.result,
      t.test = t.result,
      p.value = t.result$p.value
    ))
  }
}

#' Analyze Gene Expression Between Groups
#'
#' Performs a comprehensive statistical analysis comparing gene expression
#' between two groups. Integrates parametric and nonparametric testing,
#' effect size estimation, bootstrapped confidence intervals, and a
#' heuristic biological relevance assessment.
#'
#' @param df A data frame as produced by \code{build.analysis.df()},
#'   containing at least two columns:
#' \describe{
#'   \item{expression}{Numeric vector of gene expression values.}
#'   \item{group}{Character or factor vector with exactly two group labels.}
#' }
#' When the data frame contains multiple genes (i.e., a \code{gene} column
#' with more than one unique value), expression values from all genes are
#' pooled together. Subset the data frame to a single gene before calling
#' this function for per-gene results.
#' @param alpha Numeric. Significance threshold. Default is \code{0.05}.
#' @param n.boot Integer. Number of bootstrap resamples for the confidence
#'   interval. Default is \code{1000}.
#'
#' @return A named list with nine elements:
#' \describe{
#'   \item{p.value}{Numeric. P-value from the adaptive t-test.}
#'   \item{effect.size}{Numeric. Cohen's d.}
#'   \item{effect.size.class}{Character. Qualitative magnitude label from
#'     \code{classify.effect.size()}.}
#'   \item{confidence.interval}{Named list with \code{lower} and \code{upper}
#'     bounds for the bootstrapped CI around Cohen's d.}
#'   \item{nonparametric.p}{Numeric. P-value from the Wilcoxon rank-sum test.}
#'   \item{robustness}{Character. Agreement assessment between parametric and
#'     nonparametric tests.}
#'   \item{biological.relevance}{Character. Heuristic relevance label from
#'     \code{flag.biological.relevance()}.}
#'   \item{interpretation}{Character. Human-readable summary combining all
#'     of the above.}
#'   \item{raw}{List. Raw output from the parametric and nonparametric tests
#'     for further inspection.}
#' }
#'
#' @details
#' This function integrates multiple statistical perspectives to provide a
#' more complete picture of group differences than a p-value alone. The
#' result is designed to feed directly into \code{plot.gene.analysis()} for
#' visualization.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#' probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")
#' expr <- get.gene.expression(geo$expression, probe)
#' analysis.df <- build.analysis.df(expr, geo$phenotype, geo$gene)
#' result <- analyze.gene(analysis.df, n.boot=100)
#' cat(result$interpretation)
#' }
#'
#' @export
analyze.gene <- function(df, alpha=0.05, n.boot=1000)
{
  if(!all(c("expression", "group") %in% colnames(df)))
  {
    stop("Data frame must contain 'expression' and 'group' columns.")
  }

  if(length(unique(df$group)) != 2)
  {
    stop("The 'group' column must contain exactly two groups.")
  }

  test.results <- adaptive.t.test(df, alpha)
  p.value <- test.results$p.value

  d <- compute.effect.size(df)
  d.class <- classify.effect.size(d)

  ci <- compute.ci(df, alpha, n.boot)

  np.result <- nonparametric.test(df)
  np.p <- np.result$p.value

  if(p.value < alpha && np.p < alpha)
  {
    robustness <- "robust across parametric and nonparametric tests"
  }
  else if(p.value < alpha && np.p >= alpha)
  {
    robustness <- "sensitive to distributional assumptions"
  }
  else
  {
    robustness <- "no consistent evidence of difference"
  }

  bio.flag <- flag.biological.relevance(d, p.value, alpha)

  interpretation <- paste(
    "The difference is ",
    ifelse(p.value < alpha, "statistically significant ", "not statistically significant "),
    "with a ", d.class, " effect size.",
    "\nResult is ", robustness, ".\n",
    bio.flag, ".", sep=""
  )

  return(list(
    p.value = p.value,
    effect.size = d,
    effect.size.class = d.class,
    confidence.interval = ci,
    nonparametric.p = np.p,
    robustness = robustness,
    biological.relevance = bio.flag,
    interpretation = interpretation,
    raw = list(
      parametric = test.results,
      nonparametric = np.result
    )
  ))
}

#' Run Differential Expression Analysis Using limma
#'
#' Performs genome-wide differential expression analysis across all probes
#' using the limma linear modeling framework. This is the recommended approach
#' for microarray data from GEO, as it borrows variance information across
#' genes to produce more stable estimates than gene-by-gene t-tests.
#'
#' @param geo Named list as returned by \code{extract.expression()}, containing
#'   at minimum \code{$expression} (probe x sample matrix) and
#'   \code{$phenotype} (sample metadata data frame).
#'
#' @param condition.col Character. Name of the column in \code{geo$phenotype}
#'   to use as the grouping variable. Default is \code{"disease.state"}.
#'
#' @param adjust.method Character. P-value correction method passed to
#'   \code{limma::topTable()}. Default is \code{"BH"} (Benjamini-Hochberg FDR).
#'
#' @return A data frame (TopTable) with one row per probe, sorted by evidence
#'   of differential expression, containing:
#' \describe{
#'   \item{logFC}{Log2 fold-change between conditions.}
#'   \item{AveExpr}{Average log2 expression across all samples.}
#'   \item{t}{Moderated t-statistic.}
#'   \item{P.Value}{Raw p-value.}
#'   \item{adj.P.Val}{FDR-adjusted p-value (BH by default).}
#'   \item{B}{Log-odds that the gene is differentially expressed.}
#' }
#'
#' @details
#' Unlike running \code{analyze.gene()} on each probe individually, limma fits
#' a linear model to all probes simultaneously and applies empirical Bayes
#' shrinkage to the variance estimates. This stabilizes results for genes with
#' few observations and is the standard method for microarray DE analysis.
#'
#' The returned TopTable is the primary input for downstream functions such as
#' \code{volcano.plot()} and \code{run.gsea()}, and can be filtered by
#' \code{adj.P.Val} and \code{logFC} thresholds to define a gene signature.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
#' de.results <- run.limma.de(geo, condition.col = "disease.state")
#' head(de.results)
#' }
#'
#' @export
run.limma.de <- function(geo, condition.col="disease.state", adjust.method="BH")
{
  group <- factor(geo$phenotype[[condition.col]])
  design <- model.matrix(~ group)

  fit <- limma::lmFit(geo$expression, design)
  fit <- limma::eBayes(fit)

  return(limma::topTable(fit, adjust.method=adjust.method, number=Inf))
}

#' Adjust P-Values for Multiple Comparisons
#'
#' Applies a multiple testing correction to a vector of raw p-values, reducing
#' the false discovery rate when many genes are tested simultaneously.
#'
#' @param p.values Numeric vector of raw p-values, one per gene or probe.
#' @param method Character. Correction method passed to \code{p.adjust()}.
#'   Common options are \code{"BH"} (Benjamini-Hochberg), \code{"bonferroni"},
#'   and \code{"holm"}. Default is \code{"BH"}.
#'
#' @return A numeric vector of adjusted p-values, the same length and order
#'   as the input. For the BH method values represent the estimated false
#'   discovery rate (FDR); for Bonferroni and Holm they represent the
#'   family-wise error rate.
#'
#' @details
#' When \code{analyze.gene()} is applied across many probes in a loop, each
#' test is performed independently and p-values are not corrected for
#' multiplicity. This function should be applied to the resulting p-value
#' vector before interpreting significance. BH correction is recommended for
#' exploratory genomic analyses; Bonferroni is more conservative and suited
#' to confirmatory settings. For genome-wide analysis, prefer
#' \code{run.limma.de()} which handles correction internally.
#'
#' @examples
#' \donttest{
#' geo <- extract.expression(load.geo.soft(accession = "GDS507", log.transform = TRUE))
#' probe.ids <- find.probe.by.gene(geo$gene, c("GENE1", "GENE2", "GENE3"))
#'
#' raw.pvals <- sapply(probe.ids, function(id) {
#'   expr <- get.gene.expression(geo$expression, id)
#'   df <- build.analysis.df(expr, geo$phenotype, geo$gene)
#'   analyze.gene(df)$p.value
#' })
#' adj.pvals <- adjust.pvalues(raw.pvals, method = "BH")
#' }
#'
#' @export
adjust.pvalues <- function(p.values, method="BH")
{
  return(p.adjust(p.values, method=method))
}

#' Compute Pairwise Gene Co-expression Correlation Matrix
#'
#' Calculates pairwise correlations between a set of genes across all samples,
#' producing a symmetric correlation matrix that quantifies co-expression
#' relationships.
#'
#' @param expression.matrix Numeric matrix of gene expression values as returned
#'   by \code{extract.expression()$expression}. Rows are probes, columns are
#'   samples.
#'
#' @param probe.ids Integer vector of probe IDs to include, as returned by
#'   \code{find.probe.by.gene()}.
#' @param method Character. Correlation method: \code{"pearson"} (default),
#'   \code{"spearman"}, or \code{"kendall"}. Spearman is recommended when
#'   distributions are skewed or outliers are a concern.
#'
#' @return A symmetric numeric matrix of dimensions \code{length(probe.ids)} x
#'   \code{length(probe.ids)}, where each cell contains the pairwise correlation
#'   coefficient across all samples. Diagonal values are 1. Row and column
#'   names correspond to probe IDs.
#'
#' @details
#' Co-expression correlations capture whether two genes tend to be
#' simultaneously up- or down-regulated across samples, which can suggest
#' shared regulatory control or pathway membership. The resulting matrix is
#' the direct input for \code{plot.correlation.heatmap()}.
#'
#' @examples
#' \donttest{
#' set.seed(42)
#' expr.mat <- matrix(rnorm(400), nrow = 4, ncol = 100)
#' rownames(expr.mat) <- c(101, 102, 103, 104)
#' probe.ids <- c(101, 102, 103, 104)
#' cor.mat <- gene.correlation.matrix(expr.mat, probe.ids)
#' correlation.heatmap.plot(cor.mat, gene.names = c("BRCA1", "TP53", "MYC", "EGFR"))
#' }
#'
#' @export
gene.correlation.matrix <- function(expression.matrix, probe.ids, method="pearson")
{
  subset <- expression.matrix[which(rownames(expression.matrix) %in% probe.ids),]
  return(cor(t(subset), method=method))
}

#' Fit LASSO Regression for Multi-Gene Biomarker Discovery
#'
#' Applies cross-validated LASSO logistic regression to identify a sparse set
#' of genes that jointly predict a binary phenotype. Unlike univariate tests,
#' LASSO accounts for the joint contribution of all genes simultaneously,
#' automatically penalizing redundant predictors.
#'
#' @param expression.matrix Numeric matrix of gene expression values as returned
#'   by \code{extract.expression()$expression}. Rows are probes, columns are
#'   samples. The matrix is transposed internally so samples become rows as
#'   required by \code{glmnet}.
#'
#' @param phenotype.vector Factor or integer vector of binary phenotype labels,
#'   one per sample (e.g., \code{0} for control, \code{1} for disease). Must
#'   be the same length as the number of columns in \code{expression.matrix}.
#' @param alpha Numeric. Elastic net mixing parameter. \code{1} (default) gives
#'   pure LASSO; \code{0} gives Ridge; values between 0 and 1 give elastic net.
#' @param nfolds Integer. Number of cross-validation folds. Default is \code{10}.
#'
#' @return A \code{cv.glmnet} object. Key elements:
#' \describe{
#'   \item{lambda.min}{Lambda with minimum cross-validated error.}
#'   \item{lambda.1se}{Largest lambda within 1 SE of the minimum (sparser model).}
#'   \item{glmnet.fit}{The full sequence of fitted models across lambda values.}
#' }
#' Extract selected genes with \code{coef(fit, s = "lambda.1se")}; probes with
#' non-zero coefficients are the LASSO-selected biomarkers.
#'
#' @details
#' LASSO is suited to high-dimensional genomic data where the number of genes
#' far exceeds samples. It shrinks most coefficients to exactly zero, retaining
#' only genes with independent predictive value. This complements
#' \code{run.limma.de()} - while limma ranks genes by individual differential
#' expression, LASSO identifies the minimal subset with non-redundant joint
#' predictive signal.
#'
#' @examples
#' \donttest{
#' # Synthetic example — small expression matrix with binary outcome
#' set.seed(42)
#' expr.mat <- matrix(rnorm(200), nrow = 20, ncol = 10)
#' rownames(expr.mat) <- paste0("probe", 1:20)
#' colnames(expr.mat) <- paste0("sample", 1:10)
#' phenotype.binary <- c(0, 0, 0, 0, 0, 1, 1, 1, 1, 1)
#' lasso.fit <- fit.lasso(expr.mat, phenotype.binary)
#' coef(lasso.fit, s = "lambda.1se")
#' }
#'
#' @export
fit.lasso <- function(expression.matrix, phenotype.vector, alpha=1, nfolds=10)
{
  x <- t(expression.matrix)
  fit <- glmnet::cv.glmnet(
    x, phenotype.vector,
    alpha = alpha,
    family = "binomial",
    nfolds = nfolds
  )

  return(fit)
}

#' Run Gene Set Enrichment Analysis
#'
#' Tests whether predefined gene sets (e.g., pathways or GO terms) are
#' systematically enriched among genes ranked by differential expression,
#' using the fgseaMultilevel algorithm from \code{fgsea}.
#'
#' @param de.results Data frame as returned by \code{run.limma.de()}.
#'   Must contain a \code{logFC} column. Row names are probe IDs.
#' @param genes Data frame of gene annotations as returned by
#'   \code{extract.expression()$gene}. Must contain columns \code{"ID"} for
#'   probe identifiers and \code{"Gene symbol"} for gene symbols matching
#'   the format used by pathway databases such as MSigDB.
#' @param pathways Named list of character vectors, where each element is a
#'   gene set and names are pathway labels. MSigDB or GO gene sets in this
#'   format can be loaded via \code{msigdbr}.
#' @param min.size Integer. Minimum number of genes required in a gene set
#'   for it to be tested. Default is \code{15}.
#' @param max.size Integer. Maximum gene set size. Default is \code{500}.
#'
#' @return A data frame with one row per tested pathway, containing:
#' \describe{
#'   \item{pathway}{Gene set name.}
#'   \item{pval}{Nominal p-value.}
#'   \item{padj}{BH-adjusted p-value across all tested pathways.}
#'   \item{NES}{Normalized enrichment score. Positive values indicate
#'     enrichment among upregulated genes; negative values indicate
#'     enrichment among downregulated genes.}
#'   \item{size}{Number of genes from the pathway present in the ranked list.}
#'   \item{leadingEdge}{List-column of genes driving the enrichment score.}
#' }
#'
#' @details
#' Uses the \code{fgseaMultilevel} algorithm, which estimates p-values using
#' an adaptive multilevel splitting Monte Carlo approach. This is more
#' accurate than the original permutation-based \code{fgseaSimple} and does
#' not require a fixed permutation count.
#'
#' GSEA operates on the full ranked gene list from \code{run.limma.de()}
#' rather than a filtered subset, preserving information from all genes. Probe
#' IDs from the TopTable row names are resolved to gene symbols via the
#' \code{genes} annotation data frame before matching against pathway gene
#' sets. Probes with no gene annotation are silently dropped from the ranked
#' list, as they cannot be matched to any pathway. The \code{leadingEdge}
#' genes are strong candidates for follow-up with \code{analyze.gene()} or
#' \code{gene.analysis.plot()}.
#'
#' @examples
#' \donttest{
#' library(msigdbr)
#' geo <- extract.expression(load.geo.soft(accession = "GDS507",
#'                                                  log.transform = TRUE))
#' hallmark.df <- msigdbr(species = "Homo sapiens", category = "H")
#' pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
#' de.results <- run.limma.de(geo)
#' gsea.results <- run.gsea(de.results, geo$gene, pathways)
#' head(gsea.results[order(gsea.results$padj), ])
#' }
#'
#' @export
run.gsea <- function(de.results, genes, pathways, min.size=15, max.size=500)
{
  # Resolve probe IDs to gene symbols
  matched <- genes[which(genes$ID %in% rownames(de.results)), ]
  gene.symbols <- matched[["Gene symbol"]]
  probe.ids <- matched[["ID"]]

  ranked.genes <- de.results[probe.ids, "logFC"]
  names(ranked.genes) <- gene.symbols

  # Drop unannotated probes
  ranked.genes <- ranked.genes[names(ranked.genes) != ""]

  # Collapse duplicate gene symbols by keeping the probe with the
  # highest absolute fold change. Multiple probes per gene is common
  # on microarrays and fgsea requires unique gene names
  ranked.genes <- ranked.genes[order(abs(ranked.genes), decreasing=TRUE)]
  ranked.genes <- ranked.genes[!duplicated(names(ranked.genes))]
  ranked.genes <- sort(ranked.genes, decreasing=TRUE)

  fgsea::fgsea(
    pathways = pathways,
    stats = ranked.genes,
    minSize = min.size,
    maxSize = max.size
  )
}

---
title: "Transcriptomic Signatures of Renal Cell Carcinoma: A Visual Analytics Case Study"
author: "Spencer Treadway"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Transcriptomic Signatures of Renal Cell Carcinoma}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE,
  cache     = FALSE,
  warning   = FALSE,
  message   = FALSE,
  fig.width = 7,
  fig.height = 5
)
```

---

## 1. Research Question

Cancer biology is fundamentally a problem of gene expression. When a cell
becomes malignant, it does not simply acquire a mutation and stop there, it
undergoes a wholesale reorganization of its transcriptional state, activating
programs that promote uncontrolled growth while silencing the regulatory
mechanisms that would normally restrain it. Understanding exactly which genes
are affected, by how much, and in what coordinated patterns is the central
task of cancer transcriptomics.

This analysis addresses the following research question:

> *What transcriptomic changes characterize clear cell renal cell carcinoma,
> and can a small gene expression signature reliably distinguish tumor from
> normal kidney tissue?*

Three specific objectives follow from this question. First, to identify the
individual genes most significantly dysregulated in RCC relative to normal
kidney tissue and characterize the magnitude and statistical robustness of
those differences. Second, to determine whether differentially expressed genes
cluster into coherent biological pathways, particularly those consistent with
known RCC molecular biology. Third, to evaluate whether a sparse, LASSO-selected
gene panel can serve as a candidate diagnostic signature, establishing a
connection between population-level transcriptomics and potential clinical utility.

---

## 2. Problem Description and Hypothesis

Renal cell carcinoma (RCC) is the most common malignancy of the kidney,
accounting for approximately 90 percent of kidney cancers and an estimated 80,000
new diagnoses annually in the United States. The clear cell subtype, which
represents roughly 75 percent of RCC cases, is defined at the molecular level by
biallelic inactivation of the *VHL* (Von Hippel-Lindau) tumor suppressor gene
on chromosome 3p. VHL normally targets the transcription factor HIF1$\alpha$
(Hypoxia Inducible Factor 1-alpha) for ubiquitin-mediated degradation under
normoxic conditions. When VHL is lost, HIF1$\alpha$ constitutively accumulates
regardless of oxygen availability, driving a broad transcriptional program
that mimics chronic hypoxia. This program activates genes involved in
angiogenesis (particularly *VEGF* and its receptors), anaerobic glucose
metabolism, and cell survival, processes that collectively favor tumor
growth and invasion.

Despite this well-characterized molecular etiology, RCC is frequently
diagnosed at advanced stage because early disease is largely asymptomatic.
The "incidentaloma" pattern, discovery during imaging performed for unrelated
reasons, accounts for a growing proportion of diagnoses, but many patients
still present with metastatic disease, for which five-year survival rates
remain below 15 percent. Identifying robust molecular signatures from bulk tumor
expression data has the potential to improve early detection, subtype
classification, and patient stratification for targeted therapy.

The dataset used in this analysis, GDS507 from the NCBI Gene Expression
Omnibus, contains expression profiles from 17 clear cell RCC tumor samples
and 10 matched normal kidney cortex samples, measured on the Affymetrix
HG-U133A microarray platform (~22,000 probe sets). All patients underwent
radical or partial nephrectomy, and tumor and normal samples were collected
from the same surgical specimen where possible, reducing inter-patient
confounding.

The central hypothesis is threefold: (1) clear cell RCC will produce a
large, statistically robust differential expression signature relative to
normal kidney; (2) that signature will be enriched for gene sets related to
hypoxia, angiogenesis, and metabolic reprogramming consistent with HIF1$\alpha$
pathway activation; and (3) a small number of genes, fewer than 20, will
jointly carry sufficient discriminative information to classify tumor from
normal tissue with high accuracy.

---

## 3. Related Work

The analysis of bulk tumor gene expression data has a rich history in
computational biology, and several landmark studies have shaped both the
methods and the visual conventions used here.

The Cancer Genome Atlas (TCGA) kidney renal clear cell carcinoma (KIRC)
cohort represents the most comprehensive multi-omic characterization of
clear cell RCC to date, integrating expression, methylation, copy number,
and somatic mutation data across over 500 patients. The TCGA KIRC analysis
confirmed the centrality of VHL pathway dysregulation and identified *VEGFA*
upregulation, *CA9* (carbonic anhydrase 9) as a near-universal marker of
clear cell RCC, and broad downregulation of oxidative phosphorylation genes
as consistent molecular features of the disease. This analysis uses the same
biological hypotheses, though on a much smaller dataset, making GDS507 a
useful proof-of-concept for the methods developed in BioUtils.

The limma/voom framework used here for differential expression was introduced
by Smyth (2004) and has become the field standard for microarray analysis
specifically because of its empirical Bayes moderation of variance estimates.
Its outputs, particularly the volcano plot, have become the canonical
visualization for differential expression results across thousands of
published papers. The volcano plot is compelling as a visual because it
encodes two distinct dimensions of evidence simultaneously: the x-axis
captures biological magnitude (fold change) and the y-axis captures
statistical confidence (adjusted p-value), so that genes of genuine interest
naturally cluster in the visually distinctive upper corners. An
interactive version using plotly extends this by allowing hover-based gene
identification, which in a static figure would require labeling and quickly
becomes cluttered.

Gene Set Enrichment Analysis (GSEA), introduced by Subramanian et al.
(2005), addresses a fundamental limitation of single-gene testing: the
most statistically significant individual gene is not necessarily the most
biologically informative. GSEA operates on the full ranked gene list and
asks whether genes from a known biological pathway are systematically
enriched at either end of the ranking. The NES bar chart used here to
display GSEA results follows the visual convention established by the Broad
Institute's GSEA software and is effective as a part-to-whole visualization
because it shows, simultaneously, the direction and relative magnitude of
pathway-level responses across the entire Hallmark collection.

The LASSO biomarker approach draws on Tibshirani's original 1996 paper
and its subsequent application to genomic data by Zou and Hastie. The
cross-validated LASSO has been used extensively in cancer biomarker
discovery because it naturally handles the high-dimensional, correlated
predictor structure of expression data and produces interpretable sparse
solutions. Importantly, the genes selected by LASSO are not simply the
most differentially expressed, they are the genes with non-redundant
predictive value given each other, which is the clinically relevant
criterion for a diagnostic panel.

---

## 4. Data and Methods

```{r load}
library(BioUtils)
library(plotly)
library(msigdbr)
library(ggplot2)

geo <- extract.expression(
  load.geo.soft(accession = "GDS507", log.transform = TRUE)
)
```

All expression values are log2-transformed at import. Log transformation
is standard for Affymetrix arrays because raw intensities span several
orders of magnitude and are right-skewed; log2 transformation compresses
this range, stabilizes variance, and means that arithmetic differences
correspond to fold changes, a one-unit difference represents a doubling
of expression.

The dataset contains `r ncol(geo$expression)` samples profiled across
`r nrow(geo$expression)` probe sets. Sample composition is
`r sum(geo$phenotype$disease.state == "RCC")` RCC tumor samples and
`r sum(geo$phenotype$disease.state == "normal")` normal kidney samples.

The analytical workflow proceeds through six stages: multivariate
dimensionality reduction for quality control, genome-wide differential
expression testing, distribution analysis of top candidate genes,
correlation analysis of co-expression structure, pathway enrichment
analysis, and multi-gene predictive modeling. Each stage addresses a
distinct analytical question and employs a distinct visualization type.

---

## 5. Multivariate Analysis: Principal Component Analysis

Principal Component Analysis (PCA) is applied first, before any filtering
or testing, as a global quality control step. PCA finds the linear
combinations of the 22,000 probe variables that explain the most variance
across samples and projects each sample into this reduced space. If the
primary axis of variation in the data corresponds to the biological
comparison of interest, disease state, it suggests the transcriptional
signal is large relative to technical noise and that downstream statistical
tests will have sufficient power.

```{r pca}
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")
```

The PCA plot reveals strong separation between RCC and normal samples along
PC1, which explains the majority of variance in the dataset. This clean
separation has two important implications. First, it confirms that the
dominant source of variation in the expression matrix is biology rather than
technical factors such as batch effects, RNA quality differences, or
processing date, if technical confounders were present, we would expect
samples to cluster by processing order or batch rather than disease state.
Second, the magnitude of the PC1 separation suggests that RCC produces a
pervasive and consistent transcriptional reprogramming across tumor samples,
rather than heterogeneous, sample-specific changes. This bodes well for the
subsequent differential expression analysis and for the hypothesis that a
small gene panel can reliably classify tumor from normal tissue.

The tight clustering of normal samples relative to the somewhat wider spread
of RCC samples along PC2 is also biologically meaningful: normal kidney
tissue is a relatively homogeneous cell type (primarily proximal tubule
epithelium), whereas clear cell RCC tumors can vary in grade, stage, and
the degree of HIF1$\alpha$ pathway activation, producing slightly more variable
expression profiles within the tumor group.

---

## 6. Ranking: Genome-Wide Differential Expression

```{r de}
de.results <- run.limma.de(geo, condition.col = "disease.state")

sig <- de.results[de.results$adj.P.Val < 0.05 & abs(de.results$logFC) > 1, ]
cat("Significant DE probes (FDR < 0.05, |logFC| > 1): ", nrow(sig), "\n")
cat("Upregulated in RCC:                              ", sum(sig$logFC > 0), "\n")
cat("Downregulated in RCC:                            ", sum(sig$logFC < 0), "\n")
```

Differential expression testing using the limma empirical Bayes framework
identifies a substantial number of significantly dysregulated probes at
a 5 percent FDR threshold with a minimum 2-fold change. The asymmetry between
upregulated and downregulated genes is itself biologically informative.
RCC is notable among cancers for producing extensive downregulation of
normal kidney function genes alongside the activation of oncogenic programs,
the proximal tubule cells that give rise to clear cell RCC normally
express high levels of metabolic, transport, and detoxification genes that
are systematically silenced during malignant transformation. This pattern
of "lineage infidelity," the loss of the transcriptional identity of the
cell of origin, is reflected in the balance of up- and downregulated probes.

---

## 7. Deviation Analysis: Interactive Volcano Plot

The volcano plot is the canonical visualization for differential expression
results because it simultaneously encodes two orthogonal dimensions of
evidence. The horizontal axis represents the magnitude of change (log2
fold change), and the vertical axis represents the statistical confidence
in that change (-log10 adjusted p-value). Genes of genuine biological
interest occupy the upper corners, large magnitude changes with high
statistical confidence, while uninformative probes cluster near the
origin. The dashed threshold lines at |logFC| = 1 and FDR = 0.05 define
the boundaries used throughout this analysis.

```{r volcano_static}
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)
```

The static volcano plot communicates the global structure of the
differential expression result, the sheer number of significantly
dysregulated probes and the approximate symmetry (or asymmetry) of
upregulation versus downregulation. The interactive version below extends
this by enabling identification of specific genes by hovering, which is
particularly valuable for exploring the biologically interesting probes
in the upper corners that would be too numerous to label in a static figure.

```{r volcano_plotly}
de.plot <- de.results
de.plot$neg.log10.fdr <- -log10(de.plot$adj.P.Val)
de.plot$status <- "Not Significant"
de.plot$status[de.plot$adj.P.Val < 0.05 & de.plot$logFC > 1] <- "Upregulated"
de.plot$status[de.plot$adj.P.Val < 0.05 & de.plot$logFC < -1] <- "Downregulated"

de.plot$gene <- ""
top.idx <- order(de.plot$adj.P.Val)[1:50]
de.plot$gene[top.idx] <- get.gene.name(
  geo$gene, rownames(de.plot)[top.idx], use.symbols = TRUE
)

plotly::plot_ly(
  data = de.plot,
  x = ~logFC,
  y = ~neg.log10.fdr,
  color = ~status,
  colors = c("Upregulated" = "firebrick",
             "Downregulated" = "steelblue",
             "Not Significant" = "grey70"),
  text = ~paste0("Gene: ", gene,
                 "<br>logFC: ", round(logFC, 2),
                 "<br>FDR: ", signif(adj.P.Val, 3)),
  hoverinfo = "text",
  type = "scatter",
  mode = "markers",
  marker = list(size = 4, opacity = 0.6)
) %>%
  plotly::layout(
    title = "Volcano Plot: RCC vs Normal Kidney (Interactive, hover for gene labels)",
    xaxis = list(title = "Log2 Fold Change", zeroline = TRUE),
    yaxis = list(title = "-Log10 Adjusted P-Value", zeroline = FALSE),
    shapes = list(
      list(type = "line", x0 = 1,  x1 = 1,
           y0 = 0, y1 = max(de.plot$neg.log10.fdr, na.rm = TRUE),
           line = list(dash = "dot", color = "black")),
      list(type = "line", x0 = -1, x1 = -1,
           y0 = 0, y1 = max(de.plot$neg.log10.fdr, na.rm = TRUE),
           line = list(dash = "dot", color = "black"))
    )
  )
```

Hovering over the most extreme probes in the upper right, the most
significantly upregulated genes in RCC, reveals candidates consistent
with HIF1$\alpha$ pathway activation. Genes such as *CA9* (carbonic anhydrase 9),
which is directly transcribed by HIF1$\alpha$ and is one of the most
well-validated markers of clear cell RCC, and *VEGFA*, the master regulator
of tumor angiogenesis, appear prominently. The upper left corner,
significantly downregulated in RCC, contains genes involved in normal
proximal tubule function, particularly solute transporters and metabolic
enzymes, reflecting the loss of normal kidney cell identity during
oncogenesis.

The interactive format is particularly valuable for datasets of this scale.
With over 22,000 probes, labeling even the top 100 candidates in a static
figure produces an unreadable overlay. The hover interface allows a reader
to interrogate individual data points of interest without visual clutter,
which is a meaningful advantage of plotly over static ggplot2 for
exploratory genomic visualization.

---

## 8. Distribution Analysis: Top Gene Expression Profiles

Having identified the most significantly differentially expressed genes at
the genome-wide level, we now examine the distribution of expression values
within each group for the top candidates. This moves from the population-level
summary of the volcano plot to the sample-level detail that reveals how
cleanly each gene separates tumor from normal tissue and whether the
difference is driven by a consistent shift across all samples or by a small
number of extreme outliers.

```{r top_candidates}
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))
top.symbols <- get.gene.name(geo$gene, top.probes, use.symbols = TRUE)
valid <- which(top.symbols != "")
top.probes <- top.probes[valid]
top.symbols <- top.symbols[valid]

expr.mat <- get.gene.expression(geo$expression, top.probes)
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)

gene.analysis.plot(df.multi)
```

The faceted violin plot reveals meaningful heterogeneity among the top
candidates that is invisible in the volcano plot. Some genes show tight,
well-separated distributions in both groups, these are the strongest
candidates for robust biomarkers because their discriminative signal is
consistent across patients. Others show broader within-group distributions,
particularly in the RCC group, suggesting either biological heterogeneity
in the degree of pathway activation across tumors or the influence of
tumor purity differences (the proportion of the biopsy that is genuinely
tumor versus admixed normal stroma).

The violin shape itself is informative beyond the boxplot summary. A
bimodal violin in the RCC group, two peaks rather than one, would
suggest the presence of a molecular subgroup within the tumor samples,
potentially reflecting grade differences or the presence of a subset of
tumors with particularly aggressive HIF1$\alpha$ activation. Asymmetric tails
indicate the presence of outlier samples that may warrant investigation.

### 8.1 Single-Gene Deep-Dive

```{r single}
df.single <- df.multi[df.multi$gene == top.symbols[1], ]
gene.analysis.plot(df.single)
result <- analyze.gene(df.single)
cat(result$interpretation)
```

The single-gene analysis integrates the full statistical apparatus of
BioUtils. The adaptive t-test automatically selects between Welch's and
Student's formulations based on a prior variance equality test, avoiding
the common error of assuming homogeneity of variance across disease states
in expression data. The bootstrapped 95 percent confidence interval around Cohen's
d quantifies our uncertainty about the true effect size given the modest
sample size, a wide interval is an honest acknowledgment that 27 samples,
while adequate for genome-wide testing with limma's variance moderation, is
a relatively small basis for precise effect size estimation.

The nonparametric Wilcoxon rank-sum test provides a robustness check
against the t-test's distributional assumptions. When both tests agree on
significance, we have stronger evidence that the result is not an artifact
of distributional shape. A Cohen's d in the "large" range (above 0.8)
for the top candidate confirms that this is not merely statistical
significance inflated by sample size, the standardized difference between
tumor and normal expression is substantial by any practical standard.

---

## 9. Correlation Analysis: Gene Co-expression Structure

Individual gene analysis treats each gene independently, but genes operate
in networks. Genes under common regulatory control, sharing a transcription
factor, residing in the same pathway, or participating in the same protein
complex, tend to show correlated expression patterns across samples. Mapping
these co-expression relationships among our top candidates provides a window
into the regulatory architecture underlying the differential expression signal.

```{r coexp}
cor.mat <- gene.correlation.matrix(
  geo$expression, top.probes, method = "pearson"
)
correlation.heatmap.plot(cor.mat, gene.names = top.symbols)
```

The correlation heatmap with hierarchical clustering reveals the co-expression
structure among the top differentially expressed genes. Genes that cluster
together, appearing as red blocks on the diagonal, share expression
patterns across the 27 samples, suggesting common regulatory drivers.

Several patterns are worth examining. Strong positive co-expression among
known HIF1$\alpha$ targets (*CA9*, *VEGFA*, and other hypoxia-response genes) would
confirm that HIF1$\alpha$ pathway activation is a coherent, coordinated program
in this dataset rather than a collection of independently dysregulated genes.
Anti-correlated pairs, where one gene is high when the other is low,
are equally interesting, as they may reflect reciprocal regulatory
relationships or the activation of one pathway at the expense of another.

The hierarchical clustering dendrogram on both axes groups genes by
expression similarity. When two genes cluster tightly together, it means
their expression trajectories across the 27 samples are nearly identical,
they rise and fall together. From a biomarker perspective, tightly clustered
genes are largely redundant: including both in a diagnostic panel adds little
new information beyond what either gene alone provides. This observation
motivates the LASSO analysis in Section 11, which explicitly selects for
non-redundant predictors.

---

## 10. Part-to-Whole Analysis: Pathway Enrichment

The preceding analyses identify individual genes. This section asks a
higher-order question: do those genes belong to coherent biological programs,
and which programs are most systematically dysregulated in RCC? Gene Set
Enrichment Analysis (GSEA) addresses this by testing whether the genes in
each curated biological pathway cluster at the top or bottom of the full
differential expression ranking, rather than asking only about a pre-filtered
significant gene list.

The Hallmark gene sets from MSigDB are used here. The 50 Hallmark sets
represent carefully curated, non-redundant biological states that capture
well-defined cellular programs, they were specifically designed to reduce
the multiple testing burden and redundancy that plagues analyses using the
full GO or KEGG databases. Each set represents a coherent biological process
whose member genes were assembled by expert curation and statistical
refinement.

```{r gsea}
hallmark.df <- msigdbr(species = "Homo sapiens", category = "H")
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
gsea.results <- run.gsea(de.results, geo$gene, pathways)
gsea.results <- gsea.results[order(gsea.results$padj), ]
top.pathways <- head(gsea.results[!is.na(gsea.results$padj), ], 15)
```

```{r gsea_plot, fig.height=7}
top.pathways$short.name <- gsub("HALLMARK_", "", top.pathways$pathway)
top.pathways$direction <- ifelse(top.pathways$NES > 0,
                                 "Upregulated in RCC",
                                 "Downregulated in RCC")

ggplot2::ggplot(
  top.pathways,
  ggplot2::aes(
    x = reorder(short.name, NES),
    y = NES,
    fill = direction
  )
) +
  ggplot2::geom_col() +
  ggplot2::coord_flip() +
  ggplot2::scale_fill_manual(
    values = c("Upregulated in RCC" = "firebrick",
               "Downregulated in RCC" = "steelblue")
  ) +
  ggplot2::geom_hline(yintercept = 0, linetype = "solid", color = "black") +
  ggplot2::labs(
    title = "Hallmark Pathway Enrichment in RCC vs Normal Kidney",
    subtitle = "Bars extend from zero; direction indicates enrichment in RCC (red) or normal (blue)",
    x = NULL,
    y = "Normalized Enrichment Score (NES)",
    fill = NULL
  ) +
  ggplot2::theme_minimal() +
  ggplot2::theme(
    plot.title = ggplot2::element_text(hjust = 0.5, size = 13),
    plot.subtitle = ggplot2::element_text(hjust = 0.5, size = 9,
                                          color = "grey40"),
    legend.position = "bottom"
  )
```

The NES bar chart functions as a part-to-whole visualization at the pathway
level: each bar represents one of the 50 Hallmark programs, and the chart
as a whole shows the relative contribution of each pathway to the overall
transcriptional difference between RCC and normal kidney. The deviation from
zero is the central visual encoding, bars extending to the right represent
programs activated in RCC, while bars extending to the left represent
programs suppressed relative to normal kidney.

The results are strongly consistent with known RCC biology. Pathways with
positive NES, enriched among upregulated genes in RCC, are expected to
include **HYPOXIA**, **ANGIOGENESIS**, **GLYCOLYSIS**, and
**MYC_TARGETS**, all of which are direct or downstream consequences of
constitutive HIF1$\alpha$ activation following VHL loss. Hypoxia response genes
are among HIF1$\alpha$'s primary transcriptional targets; angiogenesis is driven
by HIF1$\alpha$-induced VEGF secretion; and the metabolic switch to glycolysis
(the Warburg effect) is a hallmark of HIF1$\alpha$-active tumors.

Pathways with negative NES, enriched among downregulated genes, are
equally revealing. Suppression of **OXIDATIVE_PHOSPHORYLATION** and
**FATTY_ACID_METABOLISM** pathways is consistent with the metabolic
reprogramming of clear cell RCC: the tumor abandons the oxidative
metabolism characteristic of the proximal tubule cell of origin in favor
of glycolytic energy production. Downregulation of
**EPITHELIAL_MESENCHYMAL_TRANSITION** (if present) would contrast with
many other carcinomas and reflect the specific biology of kidney cancer,
while suppression of immune pathway gene sets might reflect immune
exclusion mechanisms common in RCC.

The leading edge genes, those most responsible for each pathway's
enrichment score, represent the highest-priority candidates for follow-up
single-gene analysis and are the most likely to appear in the LASSO
biomarker panel in the next section.

---

## 11. Promising Trends: Multi-Gene Biomarker Discovery

The final analytical stage asks a translational question: given everything
we know about which genes are differentially expressed and which pathways
are dysregulated, can a small set of genes serve as a practical diagnostic
signature? This is not simply asking which genes are most significant, it
is asking which genes carry non-redundant predictive information when
considered jointly. This distinction matters enormously in a clinical
context, where a 3-gene RT-PCR panel is deployable and a 3,000-gene array
is not.

LASSO logistic regression is the appropriate tool for this question. Applied
to the full expression matrix, all ~22,000 probes competing simultaneously
to predict disease state, LASSO shrinks the coefficient of most genes to
exactly zero through L1 regularization, retaining only those with
independent discriminative signal. The penalty strength is tuned by
10-fold cross-validation, and the final model at `lambda.1se` represents
the sparsest solution whose cross-validated accuracy is within one standard
error of the optimum, a deliberate bias toward parsimony that tends to
produce more generalizable panels.

```{r lasso}
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)

coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected <- coef.mat[coef.mat[, 1] != 0, , drop = FALSE]
n.selected <- nrow(selected) - 1  # subtract intercept

cat("Number of genes selected by LASSO:", n.selected, "\n\n")
print(round(selected, 4))
```

```{r lasso_plot}
# Visualize the cross-validation curve to show model selection
plot(lasso.fit, main = "LASSO Cross-Validation Curve")
```

The cross-validation curve plots mean classification error against the
log-regularization penalty (log lambda), with the two vertical dashed
lines marking `lambda.min` (the penalty minimizing CV error) and
`lambda.1se` (our chosen, more parsimonious model). The relatively flat
error curve across a range of lambda values is reassuring, it means the
model is not highly sensitive to the exact penalty chosen, and that the
sparse `lambda.1se` solution performs nearly as well as the more complex
`lambda.min` solution.

The LASSO-selected genes represent a candidate diagnostic signature for
clear cell RCC. Each selected probe was chosen because it provides
classification information not already captured by any other probe in the
panel, a form of automatic feature selection that is far more principled
than simply taking the top N most significant genes from the limma
TopTable. Notably, the selected genes are not necessarily those with the
largest individual fold changes. A gene with a moderate fold change but
expression highly specific to RCC (low variance in normal tissue) may be
more valuable to the classifier than a gene that is more dramatically
changed but highly variable across both groups.

Cross-referencing the LASSO-selected probes against the GSEA leading edge
genes from Section 10 would identify which pathway members carry the most
independent predictive signal, a natural next step for prioritizing genes
for experimental validation.

---

## 12. Conclusion

This analysis demonstrates that clear cell renal cell carcinoma produces a
large, biologically coherent, and statistically robust transcriptional
signature relative to normal kidney tissue. The principal findings across
the six analytical stages are consistent and mutually reinforcing.

PCA confirmed that disease state is the dominant source of transcriptional
variation in the GDS507 dataset, with clean separation of tumor and normal
samples along the first principal component. This was a necessary precondition
for all subsequent analyses, a dataset dominated by technical noise or
batch effects would have required correction before meaningful biological
inference was possible.

Genome-wide differential expression analysis identified a large number of
significantly dysregulated probe sets at a 5 percent FDR threshold, with
asymmetric representation of upregulated versus downregulated genes
reflecting both HIF1$\alpha$ pathway activation and the loss of normal proximal
tubule cell identity. The interactive volcano plot demonstrated the value
of exploratory, hover-capable visualization for high-dimensional genomic
data, allowing rapid identification of specific genes of interest without
the visual clutter of static label overlays.

Distribution analysis of the top candidates confirmed that the most
significant genes show large, consistent effect sizes (Cohen's d in the
large range) that are robust across both parametric and nonparametric
tests. The bootstrapped confidence intervals quantify the uncertainty
inherent in a 27-sample study and provide an honest basis for interpreting
the precision of effect size estimates.

Co-expression analysis revealed coordinated regulatory structure among the
top candidates, with clusters of genes sharing expression trajectories
consistent with common transcriptional drivers. This structure motivates
the LASSO selection in the final stage, tightly co-expressed genes are
redundant predictors, and a clinically useful panel must account for this.

Pathway enrichment analysis confirmed the biological coherence of the
differential expression signature. The enrichment of hypoxia, angiogenesis,
and glycolysis pathways among upregulated genes, and the suppression of
oxidative phosphorylation and fatty acid metabolism among downregulated
genes, is precisely consistent with constitutive HIF1$\alpha$ activation following
VHL loss, the canonical molecular lesion of clear cell RCC. This
concordance between data-driven results and prior biological knowledge
substantially strengthens confidence in the analytical pipeline.

Finally, LASSO regression identified a sparse gene panel that jointly
discriminates RCC from normal tissue. The number of selected genes
demonstrates that a small, deployable molecular signature exists within
the large differential expression signal, supporting the potential
translational utility of expression-based RCC diagnostics.

Several important limitations of this analysis should be acknowledged.
The sample size of 27 (17 tumor, 10 normal) is modest for biomarker
discovery, and the LASSO cross-validation error estimates will have high
variance at this scale. All samples derive from a single study with a
single array platform, and the signature has not been validated in an
independent cohort, a critical next step before any clinical claims could
be made. The analysis also treats RCC as a binary category, ignoring tumor
grade, stage, and the molecular heterogeneity within the clear cell subtype
that would be relevant for treatment stratification.

Future directions include validation of the LASSO panel in independent GEO
datasets (several GDS entries contain RCC expression data), extension to
multi-class analysis incorporating papillary and chromophobe RCC subtypes,
integration with survival data to identify prognostic rather than diagnostic
signatures, and application of the BioUtils framework to RNA-seq data,
which has largely superseded microarrays in contemporary transcriptomics
but requires additional preprocessing (normalization, variance stabilization)
before the same statistical framework can be applied.

---
title: "BioUtils: A Case Study in Transcriptomic Analysis of Renal Cell Carcinoma"
author: "Spencer Treadway"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BioUtils: A Case Study in Transcriptomic Analysis of Renal Cell Carcinoma}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(
  collapse  = TRUE,
  comment   = "#>",
  eval      = TRUE,
  cache     = FALSE,
  warning   = FALSE,
  message   = FALSE,
  fig.width = 7,
  fig.height = 5
)
```

## Overview

Understanding which genes are expressed differently in diseased versus healthy
tissue is one of the central questions of modern molecular biology. When a cell
becomes cancerous, hundreds or thousands of genes may change their activity
levels, some becoming dramatically more active, others effectively silenced.
Identifying these changes is essential for discovering what drives the disease,
finding diagnostic biomarkers, and ultimately developing targeted therapies.

This vignette walks through a complete transcriptomic analysis of **renal cell
carcinoma (RCC)**, the most common form of kidney cancer, using publicly
available gene expression microarray data from the NCBI Gene Expression Omnibus
(GEO). The dataset, **GDS507**, contains expression profiles from 17 samples, a
mix of clear cell RCC tumor tissue and matched normal kidney tissue, measured
across ~22,000 probe sets on an Affymetrix HG-U133A microarray.

We will use the BioUtils package to carry out the full analytical workflow:
data import, quality control, differential expression analysis, visualization,
co-expression analysis, and multi-gene predictive modeling. Each step is
explained in terms of both what the code is doing and what it means
biologically.

---

## Part 1: Data Import and Quality Control

### 1.1 What Is a Microarray?

Before we load data, it helps to understand what we are working with. A DNA
microarray is a glass slide or chip printed with tens of thousands of short DNA
sequences called **probes**, each designed to bind to the RNA transcribed from a
specific gene. When a biological sample is processed and washed over the chip,
messenger RNA from active genes hybridizes to its matching probe and emits a
fluorescent signal. The brightness of each spot reflects how actively that gene
is being transcribed, its **expression level**.

The result is a table of ~22,000 numbers per sample, one for each probe on the
chip, representing the transcriptional activity of the cell at the moment the
sample was collected. Our job is to extract biologically meaningful signal from
this high-dimensional data.

### 1.2 Loading the Dataset

The GEO archive distributes datasets in SOFT format files. BioUtils reads these
directly using `load.geo.soft()`, which wraps GEOquery to parse the file and
return a structured `ExpressionSet` object:

```{r load}
library(BioUtils)

eset <- load.geo.soft("", "GDS507", log.transform = TRUE)
```

The `log.transform = TRUE` argument is important here. Affymetrix arrays report
raw intensity values that span several orders of magnitude and are right-skewed.
Applying a log2 transformation compresses this range, makes the data more
symmetric, and means that differences in expression become differences in fold
change, a 1-unit increase on the log2 scale represents a doubling of expression.
This is standard preprocessing for Affymetrix data and is necessary before any
statistical testing.

### 1.3 Extracting the Data Components

`extract.expression()` decomposes the `ExpressionSet` into the three components
used throughout BioUtils:

```{r extract}
geo <- extract.expression(eset)
```

This returns a named list with:

- **`geo$expression`** - a 22,645 by 17 matrix of log2-transformed probe
  intensities. Each row is a probe (a specific DNA sequence on the chip), each
  column is a patient sample.
- **`geo$phenotype`** - a data frame of sample metadata. Most importantly for
  us, the `disease.state` column identifies each sample as either `"RCC"` or
  `"normal"`.
- **`geo$gene`** - a data frame of probe annotations mapping each probe ID to
  the gene it targets (gene symbol, full title, chromosomal location, etc.).

### 1.4 Principal Component Analysis for Quality Control

With 22,000 variables and 17 samples, the data is far too high-dimensional to
inspect directly. **Principal Component Analysis (PCA)** is the standard first
step for understanding the global structure of the dataset. It finds the axes
of maximum variance in the data and projects each sample onto these axes, so
that similar samples cluster together in a 2D plot.

Before any analysis, we want to confirm that the largest source of variation in
the data is the biological signal we care about, disease state, rather than
technical artifacts like batch effects or sample processing date.

```{r pca}
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")
```

`pca.plot()` runs PCA on the transposed expression matrix (so samples are the
observations and probes are the variables), extracts the first two principal
components, and plots each sample as a point colored by disease state.

A well-separated PCA plot, with RCC and normal samples forming distinct clusters,
tells us that the transcriptional differences between tumor and normal tissue are
large enough to dominate the global variance structure of the dataset. This is a
reassuring quality control check: it means the biology is strong and our
statistical tests should have good power to detect it. If instead the samples
were mixed randomly or clustered by something unrelated (a technician ID, a
plate number), we would need to investigate and correct for that before
proceeding.

---

## Part 2: Genome-Wide Differential Expression

### 2.1 The Statistical Challenge

We want to know, for each of the 22,645 probes, whether the average expression
level differs significantly between RCC and normal tissue. The naive approach,
run a t-test on each probe separately, has a fundamental problem: if you run
22,645 independent tests at a 5% significance threshold, you expect to get
roughly 1,132 false positives by chance alone, even if nothing is truly
different. This is the **multiple testing problem**.

### 2.2 Running limma

BioUtils uses the `limma` (Linear Models for Microarray data) framework for
differential expression analysis, which solves both problems elegantly:

```{r limma}
de.results <- run.limma.de(geo, condition.col = "disease.state")
```

Rather than testing each gene independently, `limma` fits a linear model to all
probes simultaneously. Its key innovation is **empirical Bayes moderation**: it
borrows variance information across all genes to produce a stabilized estimate
of within-group variability for each probe. Genes with few samples or noisy
measurements benefit from this pooled information, which prevents genes with
accidentally small variance estimates from generating artificially inflated
t-statistics.

The result, called a **TopTable**, is a data frame with one row per probe,
sorted by evidence of differential expression. The most important columns are:

- **`logFC`** - the log2 fold change between RCC and normal tissue. A value of
  2 means RCC samples express that gene 4-fold higher on average. A value of
  -1 means expression is halved in RCC.
- **`adj.P.Val`** - the p-value after **Benjamini-Hochberg correction** for
  multiple testing. This controls the **false discovery rate (FDR)**: an
  `adj.P.Val` of 0.05 means we expect 5% of the probes we call significant to
  be false positives, rather than 5% of *all* tests to be wrong.

```{r inspect_de}
# Look at the top 10 most significantly differentially expressed probes
head(de.results[order(de.results$adj.P.Val), ], 10)
```

### 2.3 The Volcano Plot

A volcano plot is the standard visualization for a differential expression
result set. It places every probe in a two-dimensional space: fold change on
the x-axis (how large the difference is) and statistical significance on the
y-axis (how confident we are in that difference). The most biologically
interesting genes, large effect, high confidence, appear in the upper corners.

```{r volcano}
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)
```

The dashed lines mark our thresholds: genes must have an absolute log2 fold
change greater than 1 (a 2-fold change) AND an FDR-adjusted p-value below 0.05
to be colored as differentially expressed. Genes in red are upregulated in RCC
relative to normal; genes in blue are downregulated.

In kidney cancer, we might expect to see upregulation of genes involved in
angiogenesis (tumor blood vessel formation), glycolysis (many kidney cancers
switch their metabolism even under normal oxygen levels, the Warburg effect),
and cell proliferation, while genes involved in normal kidney function and
differentiation may be downregulated.

### 2.4 Identifying Top Candidates

We now extract the probe IDs of the most significantly differentially expressed
genes to carry forward into deeper analysis:

```{r top_probes}
# Extract the 10 most significant probe IDs (row names of the TopTable)
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))

# Resolve probe IDs to human-readable gene names
top.genes <- get.gene.name(geo$gene, top.probes)

# Some probes on the array do not map to annotated genes, remove those
annotated.mask <- which(top.genes != "")
top.probes <- top.probes[annotated.mask]
top.genes <- top.genes[annotated.mask]

cat("Top differentially expressed genes:\n")
print(top.genes)
```

`get.gene.name()` performs the reverse lookup of `find.probe.by.gene()`, given
probe IDs from the TopTable, it retrieves the human-readable gene title
from the annotation data frame. The filtering step removes probes that are
present on the array but have no gene annotation in the database (common in
older array designs).

---

## Part 3: Deep-Dive Single-Gene Analysis

The limma TopTable tells us *which* genes are differentially expressed. Now we
use BioUtils to understand *how* they are expressed, the magnitude, direction,
and robustness of each gene's difference.

### 3.1 Retrieving and Preparing Expression Data

```{r single_prep}
# Retrieve expression values for all top probes as a matrix
expr.mat <- get.gene.expression(geo$expression, top.probes)

# Build the long-format analysis data frame
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)
```

`get.gene.expression()` slices the rows of the full expression matrix
corresponding to our selected probes. `build.analysis.df()` then pivots this
wide matrix (probes by samples) into a **long-format** data frame where each row
represents one gene-sample observation, with columns for gene name, expression
value, and disease state group. This format is required by all downstream
analysis and visualization functions.

### 3.2 Multi-Gene Overview

```{r multi_plot}
# Faceted violin + boxplot across all top genes simultaneously
gene.analysis.plot(df.multi)
```

In multi-gene mode, `gene.analysis.plot()` creates a faceted panel, one
violin/boxplot per gene, allowing visual comparison of expression patterns
across the entire candidate set at once. This is useful for identifying which
genes show large, clean separation between groups and which show more
overlap or outlier-driven differences.

### 3.3 Focused Single-Gene Analysis

For the gene showing the most dramatic separation, we perform a full
statistical analysis:

```{r single_gene}
# Subset to a single gene of interest
gene.of.interest <- get.gene.name(geo$gene, top.probes[1], use.symbols=TRUE)
df.single <- df.multi[which(df.multi$gene == gene.of.interest), ]

# Annotated single-gene visualization
gene.analysis.plot(df.single)
```

In single-gene mode, `gene.analysis.plot()` runs `analyze.gene()` internally
and annotates the plot with the key statistical results.

### 3.4 Understanding `analyze.gene()`

```{r analyze}
result <- analyze.gene(df.single)
cat(result$interpretation)
```

`analyze.gene()` integrates multiple statistical perspectives into a single
object. It is worth understanding what each component contributes:

**The adaptive t-test** (`result$raw$parametric`): BioUtils first runs
`var.test()` to check whether the two groups have equal variances. If they do
not (a common occurrence in gene expression data, where disease states often
increase expression variability), it automatically uses **Welch's t-test**,
which does not assume equal variances, rather than the standard Student's
t-test. This prevents a common source of inflated false positives.

**Cohen's d** (`result$effect.size`): A p-value alone does not tell you whether
a statistically significant difference is biologically meaningful. With 17
samples, even a tiny difference in means can reach statistical significance.
Cohen's d is a **standardized effect size**, the difference in group means
divided by the pooled standard deviation, that puts the magnitude of the
difference in context. A d of 0.2 is small; 0.5 is moderate; 0.8 or above is
large. In a clinical context, genes with small effect sizes are unlikely to
serve as robust biomarkers even if they are highly significant.

**Bootstrapped confidence interval** (`result$confidence.interval`): The
confidence interval around Cohen's d quantifies our uncertainty about the true
effect size given the sample size. A narrow CI (e.g., [0.8, 1.2]) means we
have a precise estimate; a wide CI (e.g., [0.1, 1.5]) means the true effect
could be anywhere from negligible to very large, and we should be cautious
about strong claims.

**The Wilcoxon rank-sum test** (`result$nonparametric.p`): The t-test assumes
that expression values are roughly normally distributed within each group.
Gene expression data often violates this assumption, particularly when one
group has outlier samples. The Wilcoxon test makes no distributional
assumptions and provides an independent check. When both tests agree
(`result$robustness`), we can be more confident in the result. When they
disagree, t-test significant, Wilcoxon not, the result may be driven by
distributional differences rather than a genuine shift in the population mean.

```{r interpret}
# Inspect individual components
cat("Effect size (Cohen's d):", round(result$effect.size, 3), "\n")
cat("Effect size class:      ", result$effect.size.class, "\n")
cat("95% CI:                 [",
    round(result$confidence.interval$lower, 3), ",",
    round(result$confidence.interval$upper, 3), "]\n")
cat("Parametric p-value:     ", signif(result$p.value, 3), "\n")
cat("Nonparametric p-value:  ", signif(result$nonparametric.p, 3), "\n")
cat("Robustness:             ", result$robustness, "\n")
cat("Biological assessment:  ", result$biological.relevance, "\n")
```

---

## Part 4: Pathway-Level Interpretation with GSEA

So far every analysis has focused on individual genes, which ones are
differentially expressed, how large their effects are, and which ones jointly
predict disease state. But genes do not act in isolation. They participate in
coordinated **biological pathways**: cascades of molecular events that together
accomplish a cellular function. The limma TopTable tells us *which* genes
change; Gene Set Enrichment Analysis (GSEA) tells us *what those changes mean*
at the level of biology.

### 4.1 What Is a Gene Set?

A **gene set** is simply a curated list of genes known to participate in the
same biological process. The Molecular Signatures Database (MSigDB) maintains
thousands of such sets organized into collections. The most widely used
collection for disease studies is the **Hallmark** collection (`category = "H"`),
which contains 50 gene sets representing well-defined biological states,
things like "HYPOXIA", "MYC_TARGETS_V1", "INFLAMMATORY_RESPONSE", and
"EPITHELIAL_MESENCHYMAL_TRANSITION". Each set was assembled by expert curation
to be coherent and minimally overlapping with the others.

For kidney cancer specifically, we might expect to see strong enrichment of
hypoxia-related gene sets. Clear cell RCC commonly inactivates the *VHL* tumor
suppressor gene, which normally targets the HIF1$\alpha$ transcription factor for
degradation. Without VHL, HIF1$\alpha$ accumulates and drives a transcriptional
program mimicking chronic oxygen deprivation, activating genes for blood vessel
formation, glucose uptake, and anaerobic metabolism. This is one of the
best-characterized molecular signatures in all of oncology, and GSEA is an
ideal tool for detecting it.

### 4.2 Loading Gene Sets

We use the `msigdbr` package to download the Hallmark gene sets as a named
list, which is the format `run.gsea()` expects:

```{r gene_sets}
# Download Hallmark gene sets for Homo sapiens
hallmark.df <- msigdbr::msigdbr(species = "Homo sapiens", category = "H")

# Convert to a named list: pathway name -> character vector of gene symbols
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)

# Each element is a vector of gene symbols belonging to that pathway
cat("Number of Hallmark gene sets:", length(pathways), "\n")
cat("Example - first 6 genes in HALLMARK_HYPOXIA:\n")
print(head(pathways[["HALLMARK_HYPOXIA"]]))
```

### 4.3 Running GSEA

GSEA works on a **ranked gene list**, all genes in the experiment ordered from
most upregulated to most downregulated, rather than a filtered list of
significant hits. This is an important distinction from simpler enrichment
methods that only ask "are pathway genes over-represented among my significant
hits?" GSEA asks "do pathway genes cluster at the top or bottom of the full
ranked list?" which makes it sensitive to coordinated shifts across an entire
pathway even when no individual gene clears a significance threshold.

The ranking metric used by `run.gsea()` is log2 fold change from the limma
TopTable: genes at the top are the most upregulated in RCC relative to normal,
and genes at the bottom are the most downregulated.

```{r gsea}
gsea.results <- run.gsea(de.results, geo$gene, pathways, min.size = 15, max.size = 500)

# Sort by adjusted p-value and inspect the top enriched pathways
gsea.results <- gsea.results[order(gsea.results$padj), ]
print(head(gsea.results[, c("pathway", "NES", "pval", "padj", "size")], 10))
```

### 4.4 Interpreting the Results

The two most important columns in the GSEA output are:

**`NES` (Normalized Enrichment Score):** The core statistic of GSEA. A positive
NES means the gene set is enriched among genes upregulated in RCC, the genes
belonging to this pathway tend to cluster at the top of the ranked list. A
negative NES means the pathway is enriched among downregulated genes,
suppressed in RCC relative to normal tissue. The normalization accounts for
gene set size, so NES values are comparable across sets.

**`padj`:** The NES is tested for significance by permuting the gene labels and
recomputing enrichment scores under the null hypothesis that genes are randomly
distributed in the ranked list. `padj` is the resulting p-value after
Benjamini-Hochberg correction across all tested pathways.

**`leadingEdge`:** The subset of genes from the pathway that contribute most to
driving the enrichment score. These are the most biologically interesting genes
within an enriched pathway, the ones at the "leading edge" of the ranked list.
They are strong candidates for follow-up with `analyze.gene()`.

```{r gsea_followup}
# Extract the leading edge genes of the top enriched pathway
top.pathway <- gsea.results$pathway[1]
leading.genes <- unlist(gsea.results$leadingEdge[1])

cat("Top enriched pathway:", top.pathway, "\n")
cat("Leading edge genes:\n")
print(leading.genes)

# Find probes for the leading edge genes and perform single-gene deep-dives
leading.probes <- find.probe.by.gene(geo$gene, leading.genes)
leading.probes <- leading.probes[leading.probes != ""]

if(length(leading.probes) > 0)
{
  leading.expr <- get.gene.expression(geo$expression, leading.probes)
  df.leading <- build.analysis.df(leading.expr, geo$phenotype, geo$gene)
  gene.analysis.plot(df.leading)
}
```

This closes the analytical loop: GSEA identifies which biological programs are
dysregulated in RCC at the pathway level, and the leading edge genes point us
back to the individual-gene tools, `analyze.gene()` and `gene.analysis.plot()`,
to characterize the most important drivers within each pathway.

---

## Part 5: Multi-Gene Relationships

Individual gene analysis tells us about each gene in isolation. But biology is
a system, genes do not act alone. They are connected in regulatory networks,
share transcription factors, and participate in common pathways. BioUtils
provides two complementary tools for exploring these multi-gene relationships.

### 5.1 Co-expression Correlation Heatmap

Genes that are regulated together tend to show correlated expression patterns
across samples: when one goes up, the other goes up. These
**co-expression relationships** can suggest shared regulatory control,
functional partnership, or membership in the same biological pathway.

```{r coexp}
# Compute pairwise Pearson correlations between our top probes
cor.mat <- gene.correlation.matrix(geo$expression, top.probes, method = "pearson")

# Visualize with hierarchical clustering
correlation.heatmap.plot(cor.mat, gene.names = get.gene.name(geo$gene, top.probes, use.symbols=TRUE))
```

`gene.correlation.matrix()` computes the full pairwise correlation matrix
across all samples. `correlation.heatmap.plot()` then visualizes this as a
clustered heatmap, where hierarchical clustering reorders rows and columns so
that genes with similar co-expression profiles are placed next to each other.
Red cells indicate strongly co-expressed gene pairs; blue cells indicate
anti-correlated pairs (when one goes up, the other tends to go down).

Clusters of co-expressed genes visible in the heatmap often correspond to genes
in the same biological pathway or under the control of the same transcription
factor. In a renal cell carcinoma context, you might see a cluster of
angiogenesis-related genes (driven by HIF1$\alpha$ pathway activation, common in RCC)
all moving together, or a cluster of genes involved in fatty acid metabolism.

### 5.2 LASSO Regression for Predictive Biomarker Discovery

Co-expression tells us about correlations. But a separate question is: which
genes, **taken together**, best discriminate RCC from normal tissue? This is
a classification problem, and it is where `fit.lasso()` comes in.

```{r lasso}
# Encode disease state as a binary outcome variable
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)

# Fit LASSO logistic regression across all top probes simultaneously
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)

# Extract genes selected by LASSO at the 1-standard-error lambda
coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected  <- coef.mat[coef.mat[, 1] != 0, ]
print(selected)
```

**Why LASSO?** Standard logistic regression cannot be applied here: we have
more probes than samples, and many of the top genes are correlated with each
other (as shown in the heatmap). LASSO (Least Absolute Shrinkage and Selection
Operator) adds a penalty term to the logistic regression that forces most gene
coefficients to exactly zero, keeping only those with
**independent predictive value** when all other selected genes are accounted for.

`cv.glmnet()` determines the optimal penalty strength through 10-fold
cross-validation. At `lambda.1se`, the most regularized model within one
standard error of the minimum cross-validation error, LASSO returns the
sparsest model that still predicts well, which tends to be the most
interpretable and generalizable.

The genes with non-zero coefficients in the selected output are BioUtils's
answer to the question: "If you could only measure a handful of genes to
diagnose RCC from a biopsy, which ones would give you the most information?"
These are strong candidates for further validation as diagnostic biomarkers.

This is complementary to the limma DE analysis: limma identifies genes that
differ **individually** between groups, while LASSO identifies the minimal set
of genes that **jointly** carry discriminative information. A gene might be
highly significant in limma but dropped by LASSO because its information is
redundant with another gene in the panel.

---

## Part 6: The Complete Workflow

The full analysis, from raw SOFT file to multi-gene biomarker panel, can be
expressed as a coherent pipeline:

```{r full_pipeline}
library(BioUtils)

# == 1. Import =================================================================
eset <- load.geo.soft("", "GDS507", log.transform = TRUE)
geo <- extract.expression(eset)

# == 2. Quality Control ========================================================
pca.plot(geo$expression, geo$phenotype, color.by = "disease.state")

# == 3. Genome-Wide Differential Expression ====================================
de.results <- run.limma.de(geo, condition.col = "disease.state")
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)

# == 4. Select Top Candidates ==================================================
top.probes <- rownames(head(de.results[order(de.results$adj.P.Val), ], 10))
top.genes <- get.gene.name(geo$gene, top.probes)
valid.mask <- which(top.genes != "")
top.probes <- top.probes[valid.mask]
top.genes <- top.genes[valid.mask]

# == 5. Build Analysis Data Frame ==============================================
expr.mat <- get.gene.expression(geo$expression, top.probes)
df.multi <- build.analysis.df(expr.mat, geo$phenotype, geo$gene)

# == 6. Multi-Gene Visualization ===============================================
gene.analysis.plot(df.multi)

# == 7. Single-Gene Deep-Dive ==================================================
df.single <- df.multi[which(df.multi$gene == get.gene.name(geo$gene, top.probes[1], use.symbols=TRUE)), ]
gene.analysis.plot(df.single)
result <- analyze.gene(df.single)
cat(result$interpretation)

# == 8. Co-expression Analysis =================================================
cor.mat <- gene.correlation.matrix(geo$expression, top.probes, method = "pearson")
correlation.heatmap.plot(cor.mat, gene.names = get.gene.name(geo$gene, top.probes, use.symbols=TRUE))

# == 9. Pathway Enrichment =====================================================
hallmark.df <- msigdbr::msigdbr(species = "Homo sapiens", category = "H")
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
gsea.results <- run.gsea(de.results, geo$gene, pathways)
gsea.results <- gsea.results[order(gsea.results$padj), ]
print(head(gsea.results[, c("pathway", "NES", "pval", "padj")], 10))

# == 10. Multi-Gene Predictive Modeling ========================================
phenotype.binary <- ifelse(geo$phenotype$disease.state == "RCC", 1, 0)
lasso.fit <- fit.lasso(geo$expression, phenotype.binary)
coef.mat <- coef(lasso.fit, s = "lambda.1se")
selected <- coef.mat[coef.mat[, 1] != 0, ]
print(selected)
```

---

## Summary

The table below summarizes every BioUtils function used in this vignette, the
input it expects, and the output it produces:

| Function | Input | Output | Role in Workflow |
|---|---|---|---|
| `load.geo.soft()` | SOFT file path | `ExpressionSet` | Data import |
| `extract.expression()` | `ExpressionSet` | Named list | Decompose into usable components |
| `pca.plot()` | Expression matrix, phenotype | ggplot | Quality control |
| `run.limma.de()` | `geo` list | TopTable data frame | Genome-wide DE |
| `volcano.plot()` | TopTable | ggplot | DE visualization |
| `get.gene.name()` | Annotation df, probe IDs | Gene name strings | Probe to gene resolution |
| `find.probe.by.gene()` | Annotation df, gene names | Probe ID integers | Gene to probe resolution |
| `get.gene.expression()` | Expression matrix, probe IDs | Expression matrix | Slice expression data |
| `build.analysis.df()` | Expression matrix, phenotype, annotation | Long-format df | Prepare for analysis |
| `gene.analysis.plot()` | Long-format df | ggplot | Per-gene visualization |
| `analyze.gene()` | Long-format df | Results list | Full statistical analysis |
| `gene.correlation.matrix()` | Expression matrix, probe IDs | Correlation matrix | Co-expression |
| `correlation.heatmap.plot()` | Correlation matrix | pheatmap | Co-expression visualization |
| `run.gsea()` | TopTable, pathway list | GSEA results data frame | Pathway enrichment |
| `fit.lasso()` | Expression matrix, binary phenotype | cv.glmnet | Multi-gene biomarker selection |

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/analysis_prep.R
\name{build.analysis.df}
\alias{build.analysis.df}
\title{Build Analysis Data Frame}
\usage{
build.analysis.df(expr.matrix, phenotype, genes, group.col = "disease.state")
}
\arguments{
\item{expr.matrix}{Numeric matrix of gene expression values as returned by
\code{get.gene.expression()}. Rows are probe IDs, columns are sample IDs.}

\item{phenotype}{Data frame of sample metadata as returned by
\code{extract.expression()$phenotype}. Row names must correspond to the
column names of \code{expr.matrix}.}

\item{genes}{Data frame of gene annotations as returned by
\code{extract.expression()$gene}. Row indices must correspond to the probe
IDs (row names) of \code{expr.matrix}. Used to resolve probe IDs to the
human-readable gene names stored in the \code{"Gene symbol"} column.}

\item{group.col}{Character. Name of the column in \code{phenotype} to use as
the grouping variable. Default is \code{"disease.state"}.}
}
\value{
A long-format data frame with one row per gene-sample pair and three
columns:
\describe{
\item{gene}{Character. Human-readable gene name resolved from the gene
annotation data frame.}
\item{expression}{Numeric. Expression value for that gene-sample pair.}
\item{group}{Character or factor. Group label for each sample, renamed
from the \code{group.col} column in \code{phenotype} for consistency
with downstream functions.}
}
Rows with \code{NaN} or \code{NA} expression values are removed.
}
\description{
Combines a multi-gene expression matrix with sample metadata into a
long-format, analysis-ready data frame. Each row represents one gene-sample
observation, with human-readable gene names resolved from the annotation
data frame.
}
\details{
The function pivots \code{expr.matrix} from wide format (probes x samples)
to long format, merges sample metadata by sample ID, resolves probe IDs to
gene names using the annotation data frame, and selects the three columns
needed for analysis. The output is the standard input for
\code{analyze.gene()}, \code{gene.analysis.plot()}, and \code{fit.lasso()}.

Requires \code{tidyr} for the pivot step. Ensure \code{tidyr} is listed
under \code{Imports} in the package DESCRIPTION.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
probe <- find.probe.by.gene(geo$gene, c("MUC20", "ADH1A"))
expr <- get.gene.expression(geo$expression, probe)
df <- build.analysis.df(expr, geo$phenotype, geo$gene)
head(df)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_query.R
\name{get.gene.expression}
\alias{get.gene.expression}
\title{Extract Gene Expression by Probe ID}
\usage{
get.gene.expression(expression, probe.id)
}
\arguments{
\item{expression}{Numeric matrix of gene expression values as returned by
\code{extract.expression()$expression}. Rows are probe IDs, columns are
sample IDs.}

\item{probe.id}{Integer vector of one or more probe IDs as returned by
\code{find.probe.by.gene()}.}
}
\value{
A numeric matrix with one row per probe and one column per sample.
Row names are the probe IDs; column names are the sample IDs. When a
single probe ID is supplied the result is still a matrix (not a vector),
ensuring that \code{build.analysis.df()} receives a consistent input type.
}
\description{
Retrieves expression values for one or more genes from the expression matrix
using their integer probe IDs.
}
\details{
The returned matrix is the direct input for \code{build.analysis.df()}.
Using \code{as.matrix()} ensures the single-probe case returns a one-row
matrix rather than a named vector, which keeps the \code{pivot_longer}
step in \code{build.analysis.df()} consistent regardless of how many genes
are queried.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
probes <- find.probe.by.gene(geo$gene, c(
  "mucin 20, cell surface associated",
  "alcohol dehydrogenase 1A (class I), alpha polypeptide"
))
expr <- get.gene.expression(geo$expression, probes)
dim(expr)  # probes x samples
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{compute.ci}
\alias{compute.ci}
\title{Compute Confidence Interval for Effect Size}
\usage{
compute.ci(df, alpha = 0.05, n.boot = 1000)
}
\arguments{
\item{df}{A data frame containing at least two columns:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector indicating group membership.}
}}

\item{alpha}{Numeric. Significance level used to compute the confidence
interval bounds. Default is \code{0.05}, yielding a 95 percent CI.}

\item{n.boot}{Integer. Number of bootstrap resamples to perform.
Default is \code{1000}.}
}
\value{
A named list with two elements:
\describe{
\item{lower}{Lower bound of the bootstrapped confidence interval.}
\item{upper}{Upper bound of the bootstrapped confidence interval.}
}
}
\description{
Estimates a confidence interval for Cohen's d using bootstrap resampling.
This provides a measure of uncertainty around the effect size estimate,
which is especially useful when sample sizes are small or distributions
deviate from normality.
}
\details{
The function repeatedly resamples the input data with replacement and
recomputes Cohen's d for each resample. The confidence interval is
derived from the empirical distribution of bootstrapped effect sizes
using the percentile method.
}
\examples{
\donttest{
# Build a minimal example data frame
analysis.df <- data.frame(
  expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7),
  group = rep(c("normal", "RCC"), each = 4)
)
ci <- compute.ci(analysis.df, alpha = 0.05, n.boot = 100)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{gene.correlation.matrix}
\alias{gene.correlation.matrix}
\title{Compute Pairwise Gene Co-expression Correlation Matrix}
\usage{
gene.correlation.matrix(expression.matrix, probe.ids, method = "pearson")
}
\arguments{
\item{expression.matrix}{Numeric matrix of gene expression values as returned
by \code{extract.expression()$expression}. Rows are probes, columns are
samples.}

\item{probe.ids}{Integer vector of probe IDs to include, as returned by
\code{find.probe.by.gene()}.}

\item{method}{Character. Correlation method: \code{"pearson"} (default),
\code{"spearman"}, or \code{"kendall"}. Spearman is recommended when
distributions are skewed or outliers are a concern.}
}
\value{
A symmetric numeric matrix of dimensions \code{length(probe.ids)} x
\code{length(probe.ids)}, where each cell contains the pairwise correlation
coefficient across all samples. Diagonal values are 1. Row and column
names correspond to probe IDs.
}
\description{
Calculates pairwise correlations between a set of genes across all samples,
producing a symmetric correlation matrix that quantifies co-expression
relationships.
}
\details{
Co-expression correlations capture whether two genes tend to be
simultaneously up- or down-regulated across samples, which can suggest
shared regulatory control or pathway membership. The resulting matrix is
the direct input for \code{plot.correlation.heatmap()}.
}
\examples{
\donttest{
set.seed(42)
expr.mat <- matrix(rnorm(400), nrow = 4, ncol = 100)
rownames(expr.mat) <- c(101, 102, 103, 104)
probe.ids <- c(101, 102, 103, 104)
cor.mat <- gene.correlation.matrix(expr.mat, probe.ids)
correlation.heatmap.plot(cor.mat, gene.names = c("BRCA1", "TP53", "MYC", "EGFR"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{fit.lasso}
\alias{fit.lasso}
\title{Fit LASSO Regression for Multi-Gene Biomarker Discovery}
\usage{
fit.lasso(expression.matrix, phenotype.vector, alpha = 1, nfolds = 10)
}
\arguments{
\item{expression.matrix}{Numeric matrix of gene expression values as returned
by \code{extract.expression()$expression}. Rows are probes, columns are
samples. The matrix is transposed internally so samples become rows as
required by \code{glmnet}.}

\item{phenotype.vector}{Factor or integer vector of binary phenotype labels,
one per sample (e.g., \code{0} for control, \code{1} for disease). Must
be the same length as the number of columns in \code{expression.matrix}.}

\item{alpha}{Numeric. Elastic net mixing parameter. \code{1} (default) gives
pure LASSO; \code{0} gives Ridge; values between 0 and 1 give elastic net.}

\item{nfolds}{Integer. Number of cross-validation folds. Default is \code{10}.}
}
\value{
A \code{cv.glmnet} object. Key elements:
\describe{
\item{lambda.min}{Lambda with minimum cross-validated error.}
\item{lambda.1se}{Largest lambda within 1 SE of the minimum (sparser model).}
\item{glmnet.fit}{The full sequence of fitted models across lambda values.}
}
Extract selected genes with \code{coef(fit, s = "lambda.1se")}; probes with
non-zero coefficients are the LASSO-selected biomarkers.
}
\description{
Applies cross-validated LASSO logistic regression to identify a sparse set
of genes that jointly predict a binary phenotype. Unlike univariate tests,
LASSO accounts for the joint contribution of all genes simultaneously,
automatically penalizing redundant predictors.
}
\details{
LASSO is suited to high-dimensional genomic data where the number of genes
far exceeds samples. It shrinks most coefficients to exactly zero, retaining
only genes with independent predictive value. This complements
\code{run.limma.de()} - while limma ranks genes by individual differential
expression, LASSO identifies the minimal subset with non-redundant joint
predictive signal.
}
\examples{
\donttest{
# Synthetic example — small expression matrix with binary outcome
set.seed(42)
expr.mat <- matrix(rnorm(200), nrow = 20, ncol = 10)
rownames(expr.mat) <- paste0("probe", 1:20)
colnames(expr.mat) <- paste0("sample", 1:10)
phenotype.binary <- c(0, 0, 0, 0, 0, 1, 1, 1, 1, 1)
lasso.fit <- fit.lasso(expr.mat, phenotype.binary)
coef(lasso.fit, s = "lambda.1se")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/globals.R
\docType{package}
\name{BioUtils-package}
\alias{BioUtils}
\alias{BioUtils-package}
\title{BioUtils: Tools for Biological Data Analysis and Visualization}
\description{
BioUtils provides a collection of tools for preprocessing biological
datasets, performing statistical analyses, and generating visualizations.
See \code{vignette("bioutils-case-study")} for a full workflow walkthrough.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://spencertreadway.github.io/BioUtils/}
  \item \url{https://github.com/spencertreadway/BioUtils}
}

}
\author{
\strong{Maintainer}: Spencer Treadway \email{spencer.treadway08@gmail.com}

Authors:
\itemize{
  \item Spencer Treadway \email{spencer.treadway08@gmail.com}
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_query.R
\name{get.gene.name}
\alias{get.gene.name}
\title{Resolve Probe IDs to Gene Names}
\usage{
get.gene.name(genes, probe.id, use.symbols = FALSE)
}
\arguments{
\item{genes}{Data frame of gene annotations as returned by
\code{extract.expression()$gene}. Must contain columns \code{"ID"},
\code{"Gene title"}, and \code{"Gene symbol"}.}

\item{probe.id}{Integer or character vector of one or more probe IDs to
resolve, as returned by \code{find.probe.by.gene()}.}

\item{use.symbols}{Logical. If \code{FALSE} (default), returns full
descriptive gene titles from the \code{"Gene title"} column, suitable
for interpretation and reporting. If \code{TRUE}, returns short gene
symbols from the \code{"Gene symbol"} column, suitable for plot axis
labels and any downstream tool that expects standard gene symbols such
as \code{run.gsea()}.}
}
\value{
Character vector of gene names corresponding to the supplied probe
IDs. Returns empty strings \code{""} for probes with no annotation, which
should be filtered with \code{which(result != "")}.
}
\description{
Converts one or more integer probe IDs back to their human-readable gene
title strings or short gene symbols using the gene annotation data frame.
This is the inverse operation of \code{find.probe.by.gene()}.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS507",
                                        log.transform = TRUE))
de.results <- run.limma.de(geo)
top.probes <- rownames(head(de.results, 10))

# Full titles for reporting
top.titles  <- get.gene.name(geo$gene, top.probes)

# Short symbols for plot labels and GSEA
top.symbols <- get.gene.name(geo$gene, top.probes, use.symbols = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{nonparametric.test}
\alias{nonparametric.test}
\title{Perform Nonparametric Group Comparison}
\usage{
nonparametric.test(df)
}
\arguments{
\item{df}{A data frame containing at least two columns:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector with exactly two group labels.}
}}
}
\value{
A named list with two elements:
\describe{
\item{p.value}{Numeric. P-value from the Wilcoxon rank-sum test.}
\item{test}{Character. Name of the test used (\code{"Wilcoxon rank-sum"}).}
}
}
\description{
Conducts a Wilcoxon rank-sum test (Mann-Whitney U test) to compare
expression values between two groups without assuming normality.
}
\details{
This test is useful as a robustness check against parametric methods
such as the t-test, especially when data are skewed or contain outliers.
The result is used by \code{analyze.gene()} to populate the
\code{robustness} field of its return value.
}
\examples{
\donttest{
analysis.df <- data.frame(
  expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7,
                 1.5, 2.0, 1.9, 2.4),
  group      = rep(c("normal", "RCC"), each = 6)
)
res <- nonparametric.test(analysis.df)
res$p.value
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{gene.analysis.plot}
\alias{gene.analysis.plot}
\title{Plot Gene Expression with Automatic Statistical Analysis}
\usage{
gene.analysis.plot(df, alpha = 0.05, n.boot = 1000, show.points = TRUE)
}
\arguments{
\item{df}{A data frame as produced by \code{build.analysis.df()}, containing:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector with exactly two group labels.}
\item{gene}{Character vector of gene names. Optional; when present and
containing more than one unique value, enables faceted multi-gene
plotting.}
}}

\item{alpha}{Numeric. Significance level passed to \code{analyze.gene()}.
Default is \code{0.05}. Only used in single-gene mode.}

\item{n.boot}{Integer. Number of bootstrap resamples for the confidence
interval, passed to \code{analyze.gene()}. Default is \code{1000}.
Only used in single-gene mode.}

\item{show.points}{Logical. Whether to overlay jittered individual sample
points on the violin/boxplot. Default is \code{TRUE}.}
}
\value{
A \code{ggplot} object. In single-gene mode the plot includes a
text annotation with p-value, Cohen's d, and bootstrapped CI, and a
subtitle with the full interpretation string. In multi-gene mode the
plot is faceted by gene with no statistical annotation.
}
\description{
Generates an annotated visualization of gene expression between two groups.
For a single-gene data frame, the plot is annotated with statistical results
from \code{analyze.gene()}. For a multi-gene data frame, the plot is
faceted by gene and annotations are omitted, use \code{analyze.gene()} on
each gene subset for per-gene statistics.
}
\details{
Single-gene mode is triggered when the \code{gene} column is absent or
contains exactly one unique value. Multi-gene mode is triggered when
\code{gene} contains more than one unique value, producing a
\code{facet_wrap(~ gene)} layout. The plot theme is consistent across
both modes and matches the style conventions of the BioUtils package.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))

# Single-gene plot with statistical annotation
probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")
expr <- get.gene.expression(geo$expression, probe)
df <- build.analysis.df(expr, geo$phenotype, geo$gene)
gene.analysis.plot(df)

# Multi-gene faceted plot
probes <- find.probe.by.gene(geo$gene, c(
  "mucin 20, cell surface associated",
  "alcohol dehydrogenase 1A (class I), alpha polypeptide"
))
expr.multi <- get.gene.expression(geo$expression, probes)
df.multi <- build.analysis.df(expr.multi, geo$phenotype, geo$gene)
gene.analysis.plot(df.multi)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{volcano.plot}
\alias{volcano.plot}
\title{Plot Volcano Plot from Differential Expression Results}
\usage{
volcano.plot(de.results, fc.threshold = 1, fdr.threshold = 0.05)
}
\arguments{
\item{de.results}{Data frame as returned by \code{run.limma.de()}, containing
at minimum the columns \code{logFC} and \code{adj.P.Val}.}

\item{fc.threshold}{Numeric. Absolute log2 fold-change cutoff for labeling
genes as differentially expressed. Default is \code{1} (i.e., 2-fold).}

\item{fdr.threshold}{Numeric. Adjusted p-value cutoff. Default is \code{0.05}.}
}
\value{
A ggplot object showing each probe as a point with \code{logFC} on
the x-axis and \code{-log10(adj.P.Val)} on the y-axis. Points are colored
as upregulated, downregulated, or not significant. Dashed threshold lines
are drawn at the specified cutoffs.
}
\description{
Generates a volcano plot visualizing the relationship between statistical
significance and fold-change magnitude for all probes in a differential
expression analysis. Points are colored by whether they exceed user-defined
significance and fold-change thresholds.
}
\details{
The volcano plot is the standard summary visualization for a TopTable from
\code{run.limma.de()} and serves as the genome-wide complement to
\code{gene.analysis.plot()}, which visualizes a single gene. Probes that
appear in the upper corners (high fold-change, low FDR) are the strongest
candidates for follow-up with \code{analyze.gene()} or inclusion in a gene
signature for \code{run.gsea()}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_import.R
\name{extract.expression}
\alias{extract.expression}
\title{Extract Expression and Metadata from GEO Dataset}
\usage{
extract.expression(eset)
}
\arguments{
\item{eset}{An \code{ExpressionSet} object as returned by
\code{load.geo.soft()}.}
}
\value{
A named list with three elements:
\describe{
\item{expression}{Numeric matrix of gene expression values. Rows are
probe IDs, columns are sample IDs.}
\item{phenotype}{Data frame of sample metadata (e.g., disease state,
age, tissue). Row names are sample IDs matching the column names of
\code{expression}.}
\item{gene}{Data frame of probe-level gene annotations (e.g., gene
symbol, gene title, chromosomal location). Row names are probe IDs
matching the row names of \code{expression}.}
}
}
\description{
Decomposes an \code{ExpressionSet} object into its core components,
returning a named list that is used as the primary data structure
throughout BioUtils.
}
\details{
The three returned components form the basis of all downstream BioUtils
workflows. The \code{gene} data frame is passed to
\code{find.probe.by.gene()} for probe lookup, \code{expression} is
passed to \code{get.gene.expression()}, and \code{phenotype} is passed
to \code{build.analysis.df()} for group labeling.
}
\examples{
\donttest{
eset <- load.geo.soft(accession = "GDS507", log.transform = TRUE)
geo <- extract.expression(eset)
head(geo$phenotype)
dim(geo$expression)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{classify.effect.size}
\alias{classify.effect.size}
\title{Classify Effect Size}
\usage{
classify.effect.size(d)
}
\arguments{
\item{d}{Numeric. Cohen's d effect size value (signed or unsigned).}
}
\value{
Character string. One of \code{"negligible"}, \code{"small"},
\code{"moderate"}, or \code{"large"} based on the absolute value of
\code{d}, using the thresholds: negligible < 0.2 <= small < 0.5 <=
moderate < 0.8 <= large.
}
\description{
Categorizes an absolute Cohen's d value into a qualitative magnitude label
using conventional thresholds.
}
\examples{
\donttest{
classify.effect.size(0.3)   # "small"
classify.effect.size(-0.9)  # "large"
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{adaptive.t.test}
\alias{adaptive.t.test}
\title{Perform Adaptive T-Test}
\usage{
adaptive.t.test(df, alpha = 0.05)
}
\arguments{
\item{df}{A data frame containing at least two columns:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector with exactly two group labels.}
}}

\item{alpha}{Numeric. Significance level for the variance equality test
(\code{var.test()}). Default is \code{0.05}.}
}
\value{
A named list with three elements:
\describe{
\item{variance.test}{The result object from \code{var.test()} or
\code{oneway.test()}, depending on the branch taken.}
\item{t.test}{The result object from \code{t.test()}.}
\item{p.value}{Numeric. The p-value from the t-test.}
}
}
\description{
Automatically selects between Welch's and Student's t-test based on the
result of a variance equality test, then returns a unified result structure
regardless of which branch was taken.
}
\details{
If \code{var.test()} returns a p-value below \code{alpha}, variances are
considered unequal and Welch's t-test (\code{var.equal = FALSE}) is used.
Otherwise, Student's t-test (\code{var.equal = TRUE}) is applied alongside
a one-way ANOVA as a confirmatory check. The returned list is consumed
internally by \code{analyze.gene()}.
}
\examples{
\donttest{
# Build a minimal example data frame
analysis.df <- data.frame(
  expression = c(1.2, 2.3, 1.8, 2.1, 3.4, 2.9, 3.1, 2.7),
  group = rep(c("normal", "RCC"), each = 4)
)
result <- adaptive.t.test(analysis.df, alpha = 0.05)
result$p.value
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{adjust.pvalues}
\alias{adjust.pvalues}
\title{Adjust P-Values for Multiple Comparisons}
\usage{
adjust.pvalues(p.values, method = "BH")
}
\arguments{
\item{p.values}{Numeric vector of raw p-values, one per gene or probe.}

\item{method}{Character. Correction method passed to \code{p.adjust()}.
Common options are \code{"BH"} (Benjamini-Hochberg), \code{"bonferroni"},
and \code{"holm"}. Default is \code{"BH"}.}
}
\value{
A numeric vector of adjusted p-values, the same length and order
as the input. For the BH method values represent the estimated false
discovery rate (FDR); for Bonferroni and Holm they represent the
family-wise error rate.
}
\description{
Applies a multiple testing correction to a vector of raw p-values, reducing
the false discovery rate when many genes are tested simultaneously.
}
\details{
When \code{analyze.gene()} is applied across many probes in a loop, each
test is performed independently and p-values are not corrected for
multiplicity. This function should be applied to the resulting p-value
vector before interpreting significance. BH correction is recommended for
exploratory genomic analyses; Bonferroni is more conservative and suited
to confirmatory settings. For genome-wide analysis, prefer
\code{run.limma.de()} which handles correction internally.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS507", log.transform = TRUE))
probe.ids <- find.probe.by.gene(geo$gene, c("GENE1", "GENE2", "GENE3"))

raw.pvals <- sapply(probe.ids, function(id) {
  expr <- get.gene.expression(geo$expression, id)
  df <- build.analysis.df(expr, geo$phenotype, geo$gene)
  analyze.gene(df)$p.value
})
adj.pvals <- adjust.pvalues(raw.pvals, method = "BH")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{pca.plot}
\alias{pca.plot}
\title{Plot PCA of Sample Expression Profiles}
\usage{
pca.plot(
  expression.matrix,
  phenotype,
  color.by = "disease.state",
  scale = TRUE
)
}
\arguments{
\item{expression.matrix}{Matrix of gene expression values. Rows are probes,
columns are samples.}

\item{phenotype}{Data frame of sample metadata, as returned by
\code{extract.expression()$phenotype}.}

\item{color.by}{Character. Column name in \code{phenotype} to use for
point coloring. Default is \code{"disease.state"}.}

\item{scale}{Logical. Whether to scale probes to unit variance before PCA.
Default is \code{TRUE}.}
}
\value{
A ggplot object showing samples as points in PC1 vs PC2 space,
colored by the specified phenotype variable. The percentage of variance
explained is shown on each axis label.
}
\description{
Reduces the dimensionality of the full expression matrix using Principal
Component Analysis and plots samples in the first two principal component
axes, colored by a phenotype variable. Useful for quality control,
batch effect detection, and exploratory assessment of sample clustering.
}
\details{
PCA is typically applied at the whole-dataset level before any gene
filtering, making it a useful first diagnostic after \code{load.geo.soft()}
and \code{extract.expression()}. Tight clustering of samples by disease
state in PC1/PC2 space suggests a strong and detectable expression signal.
Unexpected clustering by batch, sex, or other covariates can indicate the
need for covariate adjustment in downstream \code{run.limma.de()} models.
}
\examples{
\donttest{
# Create a synthetic expression matrix (50 probes x 12 samples)
set.seed(42)
expr.mat <- matrix(rnorm(600), nrow = 50, ncol = 12)
rownames(expr.mat) <- paste0("probe", 1:50)
colnames(expr.mat) <- paste0("sample", 1:12)

# Create a matching phenotype data frame
phenotype <- data.frame(
  disease.state = rep(c("normal", "RCC"), each = 6),
  row.names = paste0("sample", 1:12)
)

pca.plot(expr.mat, phenotype, color.by = "disease.state")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{run.gsea}
\alias{run.gsea}
\title{Run Gene Set Enrichment Analysis}
\usage{
run.gsea(de.results, genes, pathways, min.size = 15, max.size = 500)
}
\arguments{
\item{de.results}{Data frame as returned by \code{run.limma.de()}.
Must contain a \code{logFC} column. Row names are probe IDs.}

\item{genes}{Data frame of gene annotations as returned by
\code{extract.expression()$gene}. Must contain columns \code{"ID"} for
probe identifiers and \code{"Gene symbol"} for gene symbols matching
the format used by pathway databases such as MSigDB.}

\item{pathways}{Named list of character vectors, where each element is a
gene set and names are pathway labels. MSigDB or GO gene sets in this
format can be loaded via \code{msigdbr}.}

\item{min.size}{Integer. Minimum number of genes required in a gene set
for it to be tested. Default is \code{15}.}

\item{max.size}{Integer. Maximum gene set size. Default is \code{500}.}
}
\value{
A data frame with one row per tested pathway, containing:
\describe{
\item{pathway}{Gene set name.}
\item{pval}{Nominal p-value.}
\item{padj}{BH-adjusted p-value across all tested pathways.}
\item{NES}{Normalized enrichment score. Positive values indicate
enrichment among upregulated genes; negative values indicate
enrichment among downregulated genes.}
\item{size}{Number of genes from the pathway present in the ranked list.}
\item{leadingEdge}{List-column of genes driving the enrichment score.}
}
}
\description{
Tests whether predefined gene sets (e.g., pathways or GO terms) are
systematically enriched among genes ranked by differential expression,
using the fgseaMultilevel algorithm from \code{fgsea}.
}
\details{
Uses the \code{fgseaMultilevel} algorithm, which estimates p-values using
an adaptive multilevel splitting Monte Carlo approach. This is more
accurate than the original permutation-based \code{fgseaSimple} and does
not require a fixed permutation count.

GSEA operates on the full ranked gene list from \code{run.limma.de()}
rather than a filtered subset, preserving information from all genes. Probe
IDs from the TopTable row names are resolved to gene symbols via the
\code{genes} annotation data frame before matching against pathway gene
sets. Probes with no gene annotation are silently dropped from the ranked
list, as they cannot be matched to any pathway. The \code{leadingEdge}
genes are strong candidates for follow-up with \code{analyze.gene()} or
\code{gene.analysis.plot()}.
}
\examples{
\donttest{
library(msigdbr)
geo <- extract.expression(load.geo.soft(accession = "GDS507",
                                                 log.transform = TRUE))
hallmark.df <- msigdbr(species = "Homo sapiens", category = "H")
pathways <- split(hallmark.df$gene_symbol, hallmark.df$gs_name)
de.results <- run.limma.de(geo)
gsea.results <- run.gsea(de.results, geo$gene, pathways)
head(gsea.results[order(gsea.results$padj), ])
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{run.limma.de}
\alias{run.limma.de}
\title{Run Differential Expression Analysis Using limma}
\usage{
run.limma.de(geo, condition.col = "disease.state", adjust.method = "BH")
}
\arguments{
\item{geo}{Named list as returned by \code{extract.expression()}, containing
at minimum \code{$expression} (probe x sample matrix) and
\code{$phenotype} (sample metadata data frame).}

\item{condition.col}{Character. Name of the column in \code{geo$phenotype}
to use as the grouping variable. Default is \code{"disease.state"}.}

\item{adjust.method}{Character. P-value correction method passed to
\code{limma::topTable()}. Default is \code{"BH"} (Benjamini-Hochberg FDR).}
}
\value{
A data frame (TopTable) with one row per probe, sorted by evidence
of differential expression, containing:
\describe{
\item{logFC}{Log2 fold-change between conditions.}
\item{AveExpr}{Average log2 expression across all samples.}
\item{t}{Moderated t-statistic.}
\item{P.Value}{Raw p-value.}
\item{adj.P.Val}{FDR-adjusted p-value (BH by default).}
\item{B}{Log-odds that the gene is differentially expressed.}
}
}
\description{
Performs genome-wide differential expression analysis across all probes
using the limma linear modeling framework. This is the recommended approach
for microarray data from GEO, as it borrows variance information across
genes to produce more stable estimates than gene-by-gene t-tests.
}
\details{
Unlike running \code{analyze.gene()} on each probe individually, limma fits
a linear model to all probes simultaneously and applies empirical Bayes
shrinkage to the variance estimates. This stabilizes results for genes with
few observations and is the standard method for microarray DE analysis.

The returned TopTable is the primary input for downstream functions such as
\code{volcano.plot()} and \code{run.gsea()}, and can be filtered by
\code{adj.P.Val} and \code{logFC} thresholds to define a gene signature.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
de.results <- run.limma.de(geo, condition.col = "disease.state")
head(de.results)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{analyze.gene}
\alias{analyze.gene}
\title{Analyze Gene Expression Between Groups}
\usage{
analyze.gene(df, alpha = 0.05, n.boot = 1000)
}
\arguments{
\item{df}{A data frame as produced by \code{build.analysis.df()},
containing at least two columns:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector with exactly two group labels.}
}
When the data frame contains multiple genes (i.e., a \code{gene} column
with more than one unique value), expression values from all genes are
pooled together. Subset the data frame to a single gene before calling
this function for per-gene results.}

\item{alpha}{Numeric. Significance threshold. Default is \code{0.05}.}

\item{n.boot}{Integer. Number of bootstrap resamples for the confidence
interval. Default is \code{1000}.}
}
\value{
A named list with nine elements:
\describe{
\item{p.value}{Numeric. P-value from the adaptive t-test.}
\item{effect.size}{Numeric. Cohen's d.}
\item{effect.size.class}{Character. Qualitative magnitude label from
\code{classify.effect.size()}.}
\item{confidence.interval}{Named list with \code{lower} and \code{upper}
bounds for the bootstrapped CI around Cohen's d.}
\item{nonparametric.p}{Numeric. P-value from the Wilcoxon rank-sum test.}
\item{robustness}{Character. Agreement assessment between parametric and
nonparametric tests.}
\item{biological.relevance}{Character. Heuristic relevance label from
\code{flag.biological.relevance()}.}
\item{interpretation}{Character. Human-readable summary combining all
of the above.}
\item{raw}{List. Raw output from the parametric and nonparametric tests
for further inspection.}
}
}
\description{
Performs a comprehensive statistical analysis comparing gene expression
between two groups. Integrates parametric and nonparametric testing,
effect size estimation, bootstrapped confidence intervals, and a
heuristic biological relevance assessment.
}
\details{
This function integrates multiple statistical perspectives to provide a
more complete picture of group differences than a p-value alone. The
result is designed to feed directly into \code{plot.gene.analysis()} for
visualization.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))
probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")
expr <- get.gene.expression(geo$expression, probe)
analysis.df <- build.analysis.df(expr, geo$phenotype, geo$gene)
result <- analyze.gene(analysis.df, n.boot=100)
cat(result$interpretation)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gene_query.R
\name{find.probe.by.gene}
\alias{find.probe.by.gene}
\title{Find Probe IDs by Gene Name}
\usage{
find.probe.by.gene(genes, gene.names)
}
\arguments{
\item{genes}{Data frame of gene annotations as returned by
\code{extract.expression()$gene}. The second column (index 2) is expected
to contain gene titles or descriptions.}

\item{gene.names}{Character vector of one or more gene names or descriptions
to search for. Matching is exact and case-sensitive.}
}
\value{
Integer vector of probe IDs corresponding to the matched genes.
The length of the vector equals the number of matches found. Returns an
empty integer vector if no matches are found.
}
\description{
Searches the gene annotation data frame to find the probe IDs corresponding
to one or more gene names or descriptions.
}
\details{
Probe IDs are the integer row names of the \code{genes} annotation data
frame. These IDs are used directly to index rows of the expression matrix
in \code{get.gene.expression()}. If multiple gene names are supplied, all
matching probe IDs are returned as a vector, which is then passed as-is to
\code{get.gene.expression()} to retrieve a multi-row expression matrix.
}
\examples{
\donttest{
geo <- extract.expression(load.geo.soft(accession = "GDS3268", log.transform = TRUE))

# Single gene
probe <- find.probe.by.gene(geo$gene, "mucin 20, cell surface associated")

# Multiple genes
probes <- find.probe.by.gene(geo$gene, c(
  "mucin 20, cell surface associated",
  "alcohol dehydrogenase 1A (class I), alpha polypeptide"
))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualization.R
\name{correlation.heatmap.plot}
\alias{correlation.heatmap.plot}
\title{Plot Gene Co-expression Correlation Heatmap}
\usage{
correlation.heatmap.plot(cor.matrix, gene.names = NULL)
}
\arguments{
\item{cor.matrix}{Numeric matrix as returned by \code{gene.correlation.matrix()}.
Must be square and symmetric with row/column names corresponding to probe IDs.}

\item{gene.names}{Character vector of gene names to use as axis labels in
place of probe IDs. Must be the same length and order as \code{cor.matrix}
rows. Default is \code{NULL}, which uses probe IDs.}
}
\value{
A \code{pheatmap} object displaying the correlation matrix with
hierarchical clustering applied to both rows and columns. Color scale
runs from blue (strong negative correlation) through white (no correlation)
to red (strong positive correlation).
}
\description{
Visualizes a gene-by-gene correlation matrix as a clustered heatmap,
revealing groups of co-expressed genes within a candidate gene set.
}
\details{
Hierarchical clustering of the correlation matrix groups genes with similar
co-expression patterns into visible blocks on the heatmap. These blocks
often correspond to genes in the same pathway or under shared regulatory
control. This function is a targeted companion to \code{gene.correlation.matrix()}
for a pre-selected gene set, and complements the genome-wide network view
produced by WGCNA.
}
\examples{
\donttest{
mat <- matrix(
  c(1.00, 0.85, 0.62, 0.91,
    0.85, 1.00, 0.74, 0.88,
    0.62, 0.74, 1.00, 0.69,
    0.91, 0.88, 0.69, 1.00),
  nrow = 4,
  dimnames = list(
    c("BRCA1", "TP53", "MYC", "EGFR"),
    c("BRCA1", "TP53", "MYC", "EGFR")
  )
)
correlation.heatmap.plot(mat, gene.names = c("BRCA1", "TP53", "MYC", "EGFR"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{compute.effect.size}
\alias{compute.effect.size}
\title{Compute Effect Size (Cohen's d)}
\usage{
compute.effect.size(df)
}
\arguments{
\item{df}{A data frame containing at least two columns:
\describe{
\item{expression}{Numeric vector of gene expression values.}
\item{group}{Character or factor vector with exactly two group labels,
as produced by \code{build.analysis.df()}.}
}}
}
\value{
Numeric. Cohen's d value. Positive values indicate the first
group (alphabetically) has higher mean expression; negative values
indicate the second group is higher.
}
\description{
Calculates the standardized mean difference between two groups.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_import.R
\name{load.geo.soft}
\alias{load.geo.soft}
\title{Load GEO Dataset from SOFT File}
\usage{
load.geo.soft(file.path = NULL, accession = NULL, log.transform = FALSE)
}
\arguments{
\item{file.path}{Character. Path to a local GEO SOFT file. If the file does
not exist and \code{accession} is provided, the dataset is downloaded
automatically.}

\item{accession}{Character. GEO accession ID (e.g., \code{"GDS507"}). Only
used when \code{file.path} does not exist. Default is \code{NULL}.}

\item{log.transform}{Logical. Whether to apply a log2 transformation to the
expression matrix during import. Default is \code{FALSE}.}
}
\value{
An \code{ExpressionSet} object for use with \code{extract.expression()}.
}
\description{
Imports a GEO dataset and converts it into a structured ExpressionSet object.
If a local SOFT file is found at \code{file.path} it is loaded directly.
Otherwise the dataset is downloaded from NCBI GEO using \code{accession} and
cached in \code{tempdir()}.
}
\examples{
\donttest{
# Download automatically if not found locally
eset <- load.geo.soft("GDS507.soft", accession = "GDS507", log.transform = TRUE)

# Download without a local file at all
eset <- load.geo.soft(NULL, accession = "GDS507", log.transform = TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stats_tests.R
\name{flag.biological.relevance}
\alias{flag.biological.relevance}
\title{Assess Biological Relevance of Gene Expression Differences}
\usage{
flag.biological.relevance(effect.size, p.value, alpha = 0.05)
}
\arguments{
\item{effect.size}{Numeric. Cohen's d or other standardized effect size.}

\item{p.value}{Numeric. P-value from a statistical test.}

\item{alpha}{Numeric. Significance threshold used for the p-value
comparison. Default is \code{0.05}.}
}
\value{
Character string. One of three heuristic labels:
\describe{
\item{"Potentially biologically meaningful"}{p < alpha and |d| > 0.5}
\item{"Statistically significant but small effect"}{p < alpha and |d| <= 0.5}
\item{"No strong evidence of biological relevance"}{p >= alpha}
}
}
\description{
Provides a heuristic interpretation of whether a statistically significant
difference in gene expression is likely to be biologically meaningful,
based on effect size magnitude and p-value.
}
\details{
These rules are heuristic and intended as guidance rather than definitive
biological conclusions. The thresholds for effect size (0.5) and
significance are conventional starting points; domain knowledge should
inform interpretation. This function is called internally by
\code{analyze.gene()} using the same \code{alpha} passed to that function.
}
\examples{
\donttest{
flag.biological.relevance(effect.size = 0.6, p.value = 0.01)
}

}
